# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

from allensdk.api.queries.rma_api import RmaApi

class ImageDownloadApi(RmaApi):
    '''HTTP Client to download whole or partial two-dimensional images from the Allen Institute
    with the SectionImage, AtlasImage and ProjectionImage Download Services.
    
    See `Downloading an Image <http://help.brain-map.org/display/api/Downloading+an+Image>`_
    for more documentation.
    '''
    def __init__(self, base_uri=None):
        super(ImageDownloadApi, self).__init__(base_uri)
    
    
    def download_section_image(self,
                               section_image_id,
                               file_path=None,
                               **kwargs):
        ''' Download whole or partial two-dimensional images
        from the Allen Institute with the SectionImage service.
        
        Parameters
        ----------
        section_image_id : integer
            Image to download.
        file_path : string, optional
            where to put it            
        downsample : int, optional
            Number of times to downsample the original image.
        quality : int, optional
            jpeg quality of the returned image, 0 to 100 (default)
        expression : boolean, optional
            True to retrieve the specified SectionImage expression mask image.
        top : int, optional
            Index of the topmost row of the region of interest.
        left :int, optional
            Index of the leftmost column of the region of interest.
        width : int, optional
            Number of columns in the output image.
        height : int, optional
            Number of rows in the output image.
        range : list of ints, optional
            Filter to specify the RGB channels.
        tumor_feature_annotation : boolean, optional
            True to retrieve the color block image for a Glioblastoma SectionImage.
        tumor_feature_boundary : boolean, optional
            True to retrieve the color boundary image for a Glioblastoma SectionImage.

        Returns
        -------
        None
            the file is downloaded and saved to the path.
            
        Notes
        -----
        'downsample=1' halves the number of pixels of the original image
        both horizontally and vertically.
        Specifying 'downsample=2' quarters the height and width values.
        
        Quality must be an integer from 0, for the lowest quality,
        up to as high as 100. If it is not specified,
        it defaults to the highest quality.
        
        Top is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.y is the default value.
        
        Left is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.x is the default value.
        
        Width is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.width is the default value. It is automatically adjusted when downsampled.
        
        Height is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.height is the default value. It is automatically adjusted when downsampled.
        
        The range parameter consists of 6 comma delimited integers
        that define the lower (0) and upper (4095) bound for each channel in red-green-blue order
        (i.e. "range=0,1500,0,1000,0,4095").
        The default range values can be determined by referring to the following fields
        on the Equalization model associated with the SectionDataSet:
        red_lower, red_uppper, green_lower, green_upper, blue_lower, blue_upper.
        For more information, see the 
        `Image Controls <http://help.brain-map.org/display/mouseconnectivity/Projection#Projection-ImageControls>`_
        section of the Allen Mouse Brain Connectivity Atlas: 
        `Projection Dataset <http://help.brain-map.org/display/mouseconnectivity/Projection>`_
        help topic.
        '''
        params = []
        
        downsample = kwargs.get('downsample', None)
        
        if downsample != None:
            params.append('downsample=%d' % (downsample))
        
        quality = kwargs.get('quality', None)
        
        if quality != None:
            params.append('quality=%d' % (quality))
        
        expression = kwargs.get('expression', None)
        
        if expression != None:
            if expression:
                params.append('expression=true')
            else:
                params.append('expression=false')
        
        # region of interest
        for roi_key in [ 'left', 'top', 'width', 'height']:
            roi_value = kwargs.get(roi_key, None)
            if roi_value != None:
                params.append('%s=%d' % (roi_key, roi_value))
        
        range_list = kwargs.get('range', None)
        
        # see http://api.brain-map.org/api/v2/data/SectionDataSet/100141599.xml?include=equalization,section_images
        if range_list:
            params.append('range=%s' % (','.join(str(r) for r in range_list)))
        
        tumor_feature_annotation = kwargs.get('tumor_feature_annotation', None)
        
        if tumor_feature_annotation != None:
            if tumor_feature_annotation:
                params.append('tumor_feature_annotation=true')
            else:
                params.append('tumor_feature_annotation=false')
        
        tumor_feature_boundary = kwargs.get('tumor_feature_boundary', None)
        
        if tumor_feature_boundary != None:
            if tumor_feature_boundary:
                params.append('tumor_feature_boundary=true')
            else:
                params.append('tumor_feature_boundary=false')
        
        if len(params) > 0:
            url_params = "?" + "&".join(params)
        else:
            url_params = ''
        
        image_url = ''.join([self.section_image_download_endpoint,
                             '/',
                             str(section_image_id),
                             url_params])
        
        if file_path == None:
            file_path = '%d.jpg' % (section_image_id)
        
        self.retrieve_file_over_http(image_url, file_path)
    
    
    def download_atlas_image(self,
                             atlas_image_id,
                             file_path=None,
                             **kwargs):
        ''' Download whole or partial two-dimensional images
        from the Allen Institute with the AtlasImage service.
        
        Parameters
        ----------
        atlas_image_id : integer
            Image to download.
        file_path : string, optional
            where to put it            
        annotation : boolean, optional
            True to retrieve the specified AtlasImage with annotations.
        atlas : integer, optional
            Atlas.ID for the annotations.
        downsample : int, optional
            Number of times to downsample the original image.
        quality : int, optional
            jpeg quality of the returned image, 0 to 100 (default)
        top : int, optional
            Index of the topmost row of the region of interest.
        left :int, optional
            Index of the leftmost column of the region of interest.
        width : int, optional
            Number of columns in the output image.
        height : int, optional
            Number of rows in the output image.
                
        Returns
        -------
        None
            the file is downloaded and saved to the path.
            
        Notes
        -----
        The P56 Adult Mouse Brain Atlas and Developing Mouse Brain Atlas
        share a common AtlasDataSet, so the Atlas.ID should be specified.
        
        'downsample=1' halves the number of pixels of the original image
        both horizontally and vertically.
        Specifying 'downsample=2' quarters the height and width values.
        
        Quality must be an integer from 0, for the lowest quality,
        up to as high as 100. If it is not specified,
        it defaults to the highest quality.
        
        Top is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.y is the default value.
        
        Left is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.x is the default value.
        
        Width is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.width is the default value. It is automatically adjusted when downsampled.
        
        Height is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.height is the default value. It is automatically adjusted when downsampled.
        
        See: `Downloading an Image - ProjectionImage Download Service <http://help.brain-map.org/display/api/Downloading+an+Image#DownloadinganImage-ProjectionImage%26nbsp%3BDownloadService>`_
        and
        `Downloading Atlas Images and Graphics <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies#AtlasDrawingsandOntologies-DownloadingAtlasImagesAndGraphics>`_.
        
        Use :py:meth:`allensdk.api.queries.ontologies_api.OntologiesApi.get_atlases_table`
        to get atlas ids programmatically.
        '''
        params = []
        
        projection = kwargs.get('projection', None)
        
        if projection != None:
            if projection == True:
                params.append('projection=true')
            else:
                params.append('projection=false')
        
        downsample = kwargs.get('downsample', None)
        
        if downsample != None:
            params.append('downsample=%d' % (downsample))
        
        quality = kwargs.get('quality', None)
        
        if quality != None:
            params.append('quality=%d' % (quality))
        
        # region of interest
        for roi_key in [ 'left', 'top', 'width', 'height']:
            roi_value = kwargs.get(roi_key, None)
            if roi_value != None:
                params.append('%s=%d' % (roi_key, roi_value))
        
        if len(params) > 0:
            url_params = "?" + "&".join(params)
        else:
            url_params = ''
        
        url = ''.join([self.projection_image_download_endpoint,
                       '/',
                       str(atlas_image_id),
                       url_params])
        
        if file_path == None:
            file_path = '%d.jpg' % (atlas_image_id)
        
        self.retrieve_file_over_http(url, file_path)
    
    
    def download_projection_image(self,
                                  projection_image_id,
                                  file_path=None,
                                  **kwargs):
        ''' Download whole or partial two-dimensional images
        from the Allen Institute with the AtlasImage service.
        
        Parameters
        ----------
        atlas_image_id : integer
            Image to download.
        file_path : string, optional
            where to put it.  default is id.jpg            
        projection : boolean, optional
            True to retrieve the specified SectionImage with projection.
        downsample : int, optional
            Number of times to downsample the original image.
        quality : int, optional
            jpeg quality of the returned image, 0 to 100 (default)
        top : int, optional
            Index of the topmost row of the region of interest.
        left :int, optional
            Index of the leftmost column of the region of interest.
        width : int, optional
            Number of columns in the output image.
        height : int, optional
            Number of rows in the output image.
        
        Returns
        -------
        None
            the file is downloaded and saved to the path.
            
        Notes
        -----
        The P56 Adult Mouse Brain Atlas and Developing Mouse Brain Atlas
        share a common AtlasDataSet, so the Atlas.ID should be specified.
        
        'downsample=1' halves the number of pixels of the original image
        both horizontally and vertically.
        Specifying 'downsample=2' quarters the height and width values.
        
        Quality must be an integer from 0, for the lowest quality,
        up to as high as 100. If it is not specified,
        it defaults to the highest quality.
        
        Top is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.y is the default value.
        
        Left is specified in full-resolution (largest tier) pixel coordinates.
        SectionImage.x is the default value.
        
        Width is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.width is the default value. It is automatically adjusted when downsampled.
        
        Height is specified in tier-resolution (desired tier) pixel coordinates.
        SectionImage.height is the default value. It is automatically adjusted when downsampled.
        
        See: `Downloading an Image - AtlasImage Download Service <http://help.brain-map.org/display/api/Downloading+an+Image#DownloadinganImage-AtlasImage%26nbsp%3BDownloadService>`_
        and
        `Downloading Atlas Images and Graphics <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies#AtlasDrawingsandOntologies-DownloadingAtlasImagesAndGraphics>`_.
        
        Use :py:meth:`allensdk.api.queries.ontologies_api.OntologiesApi.get_atlases_table`
        to get atlas ids programmatically.
        '''
        params = []
        
        annotation = kwargs.get('annotation', None)
        
        if annotation != None:
            if annotation == True:
                params.append('annotation=true')
            else:
                params.append('annotation=false')
        
        atlas = kwargs.get('atlas', None)
        
        if atlas != None:
            params.append('atlas=%d' % (atlas))
        
        downsample = kwargs.get('downsample', None)
        
        if downsample != None:
            params.append('downsample=%d' % (downsample))
        
        quality = kwargs.get('quality', None)
        
        if quality != None:
            params.append('quality=%d' % (quality))
        
        # region of interest
        for roi_key in [ 'left', 'top', 'width', 'height']:
            roi_value = kwargs.get(roi_key, None)
            if roi_value != None:
                params.append('%s=%d' % (roi_key, roi_value))
        
        if len(params) > 0:
            url_params = "?" + "&".join(params)
        else:
            url_params = ''
        
        url = ''.join([self.atlas_image_download_endpoint,
                       '/',
                       str(projection_image_id),
                       url_params])
                    
        if file_path == None:
            file_path = '%d.jpg' % (projection_image_id)
        
        self.retrieve_file_over_http(url, file_path)
    
    
    def atlas_image_query(self, atlas_id, image_type_name=None):
        '''Build the URL.
        
        Parameters
        ----------
        atlas_id : integer, optional
            request a certain record.
        image_type_name : string, optional
            if not present, the query will get it from the atlas id.
        
        Returns
        -------
        url : string
            The constructed URL
        
        Notes
        -----
        See `Downloading Atlas Images and Graphics <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies#AtlasDrawingsandOntologies-DownloadingAtlasImagesAndGraphics>`_
        for additional documentation.
        The atlas id can be found with :py:meth:`allensdk.api.queries.ontologies_api.OntologiesApi.build_atlases_query`
        '''
        rma = self
        
        stages = []
        
        if image_type_name == None:
            atlas_stage = rma.model_stage('Atlas',
                                          criteria='[id$eq%d]' % (atlas_id),
                                          only=['image_type'])
            stages.append(atlas_stage)
            
            atlas_name_pipe_stage = rma.pipe_stage('list',
                                                   parameters=[('type_name',
                                                               rma.IS,
                                                               rma.quote_string('image_type'))])
            stages.append(atlas_name_pipe_stage)
            
            image_type_name = '$type_name'
        else:
            image_type_name = rma.quote_string(image_type_name)
        
        criteria_list = ['[annotated$eqtrue],',
                         'atlas_data_set(atlases[id$eq%d]),' % (atlas_id),
                         "alternate_images[image_type$eq%s]" % (image_type_name)]
        
        atlas_image_model_stage = rma.model_stage('AtlasImage',
                                                  criteria=criteria_list,
                                                  order=['sub_images.section_number'],
                                                  num_rows='all')
        
        stages.append(atlas_image_model_stage)
        
        return self.json_msg_query(
            rma.build_query_url(stages))


import os, logging, re
import numpy as np
from preprocess_neuron import preprocess_neuron

import allensdk.core.json_utilities as ju

DATA_CONFIG_FILES = [
    "/data/mat/Corinne/GLIF_subset/data_config_files/329552531_data_config.json",
    "/data/mat/Corinne/GLIF_subset/data_config_files/476106176_data_config.json",
    "/data/mat/Corinne/GLIF_subset/data_config_files/485455285_data_config.json",
    "/data/mat/Corinne/GLIF_subset/data_config_files/369697038_data_config.json",
    "/data/mat/Corinne/GLIF_subset/data_config_files/485912047_data_config.json"
]

TEST_DIR = "/local1/stash/glif_clean/test/dictionaries/"

OUT_DIR = "/local1/stash/allensdk/allensdk/model/glif/test/preprocess"

def assert_equal(v1, v2, message, errs):
    if isinstance(v1, list) or isinstance(v1, np.ndarray):
        if not (v1 == v2).all():
            errs.append("%s: %s != %s" % (message, str(v1), str(v2)))        
    elif v1 != v2:
        if v1 is None or v2 is None:
            errs.append("%s: %s != %s" % (message, str(v1), str(v2)))
        else:
            diff = abs(v1 - v2)
            logging.info("%s: differ by %f pct" % (message, diff / v1 * 100.0))
            
            errs.append("%s: %s != %s" % (message, str(v1), str(v2)))

def test_preprocess_neuron():
    logging.getLogger().setLevel(logging.DEBUG)
    p = re.compile("(\d+)_data_config.json")
    dt = 5e-05
    bessel = { 'N': 4, 'Wn': .1 }
    cut = 0

    if not os.path.exists(OUT_DIR):
        os.makedirs(OUT_DIR)


    for data_config_file in DATA_CONFIG_FILES:
        logging.info("testing %s" % data_config_file)
        fname = os.path.basename(data_config_file)
        m = p.match(fname)

        sid = m.groups()

        test_data_file = os.path.join(TEST_DIR, "%s_preprocessed_dict.json" % sid)

        if not os.path.exists(test_data_file):
            logging.warning("no test file %s" % test_data_file)
            continue

        out_file = os.path.join(OUT_DIR, "%s_preprocessed_dict.json" % sid)

        data_config = ju.read(data_config_file)
        nwb_file = data_config["filename"]
        sweep_list = data_config["sweeps"].values()

        d = preprocess_neuron(nwb_file, sweep_list, dt, cut, bessel)

        dictionary = ju.read(test_data_file)

        ju.write(out_file, d)

        errs = []
        assert_equal(d['El'], 0.0, 'El', errs)
        assert_equal(d['El_reference'], dictionary['El']['El_noise']['measured']['mean'], 'El_reference', errs)
        assert_equal(d['deltaV'], None, 'deltaV', errs)
        assert_equal(d['dt'], dictionary['dt_used_for_preprocessor_calculations'], 'dt', errs)
        assert_equal(d['R_input'], dictionary['resistance']['R_lssq_Wrest']['mean'], 'R_input', errs)
        assert_equal(d['C'], dictionary['capacitance']['C_lssq_Wrest']['mean'], 'C', errs)
        assert_equal(d['th_inf'], dictionary['th_inf']['via_Vmeasure']['from_zero'], 'th_inf', errs)
        assert_equal(d['th_adapt'], dictionary['th_adapt']['from_95percentile_noise']['deltaV'], 'th_adapt', errs)
        assert_equal(d['spike_cut_length'], dictionary['spike_cutting']['NOdeltaV']['cut_length'], 'spike_cut_length', errs)
        assert_equal(d['spike_cutting_intercept'], dictionary['spike_cutting']['NOdeltaV']['intercept'], 'spike_cutting_intercept', errs)
        assert_equal(d['spike_cutting_slope'], dictionary['spike_cutting']['NOdeltaV']['slope'], 'spike_cutting_slope', errs)
        assert_equal(d['asc_amp_array'], dictionary['asc']['amp'], 'asc_amp_array', errs)
        assert_equal(d['asc_tau_array'], 1./np.array(dictionary['asc']['k']), 'asc_tau_array', errs)

        nlp = d['nonlinearity_parameters']
        assert_equal(nlp['line_param_RV_all'], dictionary['nonlinearity_parameters']['line_param_RV_all'], 'line_param_RV_all', errs)
        assert_equal(nlp['line_param_ElV_all'], dictionary['nonlinearity_parameters']['line_param_ElV_all'], 'line_param_ElV_all', errs)

        ta = d['threshold_adaptation']
        assert_equal(ta['a_spike_component_of_threshold'], dictionary['threshold_adaptation']['a_spike_component_of_threshold'], 'a_spike', errs)
        assert_equal(ta['b_spike_component_of_threshold'], dictionary['threshold_adaptation']['b_spike_component_of_threshold'], 'b_spike', errs) 
        assert_equal(ta['a_voltage_component_of_threshold'], dictionary['threshold_adaptation']['a_voltage_component_of_threshold'], 'a_voltage', errs)
        assert_equal(ta['b_voltage_component_of_threshold'], dictionary['threshold_adaptation']['b_voltage_component_of_threshold'], 'b_voltage', errs) 

        mlin = d['MLIN']
        assert_equal(mlin['var_of_section'], dictionary['MLIN']['var_of_section'], 'var_of_section', errs)
        assert_equal(mlin['sv_for_expsymm'],  dictionary['MLIN']['sv_for_expsymm'], 'sv_for_expsymm', errs)
        assert_equal(mlin['tau_from_AC'], dictionary['MLIN']['tau_from_AC'], 'tau_from_AC', errs)

        if len(errs) > 0:
            for err in errs:
                logging.error(err)
            raise Exception("Preprocessor outputs did not match.")
            
    

if __name__ == "__main__": test_preprocess_neuron()



# Copyright 2015-2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.


import pytest
from mock import MagicMock
from allensdk.model.biophys_sim.config import Config
from allensdk.core.json_utilities import JsonComments


@pytest.fixture
def simple_config():
    manifest = {
        'manifest': [
            { 'type': 'dir',
              'spec': 'MOCK_DOT',
              'key': 'BASEDIR'
            }],
        'biophys':
            [{ 'hoc': [ 'stdgui.hoc'] }],
    }
    
    ju = JsonComments
    ju.read_file = MagicMock(return_value=manifest)
    
    config = Config().load('config.json', False)
    
    return config


def testAccessHocFilesInData(simple_config):
    assert simple_config.data['biophys'][0]['hoc'][0] == 'stdgui.hoc'


def testManifestIsNotInData(simple_config):
    assert 'manifest' not in simple_config.data


def testManifestInReservedData(simple_config):
    assert 'manifest' in simple_config.reserved_data[0]



import os, argparse
import allensdk.core.swc as swc
    
def validate_swc(swc_file):
    """ 
    To be compatible with NEURON, SWC files must have the following properties:
        1) a single root node with parent ID '-1'
        2) sequentially increasing ID numbers
        3) immediate children of the soma cannot branch
    """
    soma_id = swc.Morphology.SOMA
    morphology = swc.read_swc(swc_file)
    # verify that there is a single root node
    num_soma_nodes = sum([ (int(c['type']) == soma_id) for c in morphology.compartment_list ])
    if num_soma_nodes != 1:
        raise Exception("SWC must have single soma compartment.  Found: %d" % num_soma_nodes)
    # sanity check 
    root = morphology.root
    if root is None:
        raise Exception("Morphology has no root node")
    # verify that children of the root have max one child
    for root_child_id in root['children']:
        root_child = morphology.compartment_index[root_child_id]
        num_grand_children = len(root_child['children'])
        if num_grand_children > 1:
            raise Exception("Child of root (%s) has more than one child (%d)" % ( root_child_id, num_grand_children ))
    # get a list of all of the ids, make sure they are unique while we're at it
    all_ids = set()
    for compartment in morphology.compartment_list:
        iid = int(compartment["id"])
        if iid in all_ids:
            raise Exception("Compartment ID %s is not unique." % compartment["id"])
        pid = int(compartment["parent"])
        if iid < pid:
            raise Exception("Compartment (%d) has a smaller ID that its parent (%d)" % (iid, pid))
        all_ids.add(iid)
        
    # sort the ids and make sure there are no gaps
    sorted_ids = sorted(all_ids)
    for i in xrange(1, len(sorted_ids)):
        if sorted_ids[i] - sorted_ids[i-1] != 1:
            raise Exception("Compartment IDs are not sequential")
    return True
    
    
    
def main():
    try:
        parser = argparse.ArgumentParser("validate an SWC file for use with NEURON")
        parser.add_argument('swc_file')
        args = parser.parse_args()
        validate_swc(args.swc_file)
    except Exception as e:
        print str(e)
        exit(1)
if __name__ == "__main__": main()



import logging
import os, argparse
import matplotlib.pyplot as plt
import h5py 
import numpy as np

from allensdk.core.brain_observatory_nwb_data_set import BrainObservatoryNwbDataSet

def movingmode_fast(x, kernelsize, y):
    """ Compute the windowed mode of an array.

    Parameters
    ----------
    x: np.ndarray
        Array to be analyzed
    kernelsize: int
        Size of the moving window
    y: np.ndarray
        Output array to store the results
    """

    histo = np.zeros(4096)
    halfsize = kernelsize/2
    for m in range (0,halfsize):
        histo[int(round(x[m]))] += 1

    mode = 1
    for n in range (2,1000):      #the range of possible mode value is < 1000
        if histo[n] > histo[mode]:
            mode = n
        
    # here initial mode is available
    for m in range (0,halfsize):
        q = int(round(x[halfsize+m]))
        histo[q] += 1
        if histo[q] > histo[mode]:
            mode = q
        y[m] = mode
    
    for m in range (halfsize,x.shape[0]-halfsize):
        p = int(round(x[m-halfsize]))
        histo[p] -= 1
        if p == mode:    #need to find possibly new mode value
            for n in range (2,1000):      #the range of possible mode value is < 1000
                if histo[n] > histo[mode]:
                    mode = n

        q = int(round(x[m+halfsize]))
        histo[q] += 1
        if histo[q] > histo[mode]:	
            mode = q

        y[m] = mode

    for m in range (x.shape[0]-halfsize,x.shape[0]):
        p = int(round(x[m-halfsize]))
        histo[p] -= 1
        if p == mode:    #need to find possibly new mode value
            for n in range (2,1000):      #the range of possible mode value is < 1000
                if histo[n] > histo[mode]:
                    mode = n

        y[m] = mode

    return 0

def movingaverage(x, kernelsize, y):
    """ Compute the windowed average of an array.

    Parameters
    ----------
    x: np.ndarray
        Array to be analyzed
    kernelsize: int
        Size of the moving window
    y: np.ndarray
        Output array to store the results
    """
    halfsize = kernelsize/2
    sumkernel = np.sum(x[0:halfsize])
    for m in range (0,halfsize):
        sumkernel = sumkernel + x[m+halfsize]
      	y[m] = sumkernel/(halfsize+m)

    sumkernel = np.sum(x[0:kernelsize])
    for m in range (halfsize,x.shape[0]-halfsize):
        sumkernel = sumkernel - x[m-halfsize] + x[m+halfsize]
      	y[m] = sumkernel/kernelsize

    for m in range (x.shape[0]-halfsize,x.shape[0]):
        sumkernel = sumkernel - x[m-halfsize]
      	y[m] = sumkernel/(halfsize-1+(x.shape[0]-m))

    return 0

def plot_onetrace(x1):
    """ Debug plotting function """
    q1 = 30000
    q2 = 60000
    q3 = 90000
    q4 = 120000

    yticks = [0.2,0.4,0.6,0.8,1.0,1.2,1.4,1.6,1.8,2.0,2.2,2.4,2.6,2.8,3.0,10.0]
    plt.subplot(4,1,1)
    plt.plot(np.arange(0,q1),x1[0:q1],'g')
    plt.ylim(0, 10)
    plt.yticks(yticks)
    plt.xlabel('frames',fontsize=18)
    plt.ylabel('DF/F',fontsize=18)

    plt.subplot(4,1,2)
    plt.plot(np.arange(q1,q2),x1[q1:q2],'g')
    plt.ylim(0, 10)
    plt.yticks(yticks)
    plt.xlabel('frames',fontsize=18)
    plt.ylabel('DF/F',fontsize=18)

    plt.subplot(4,1,3)
    plt.plot(np.arange(q2,q3),x1[q2:q3],'g')
    plt.ylim(0, 10)
    plt.yticks(yticks)
    plt.xlabel('frames',fontsize=18)
    plt.ylabel('DF/F',fontsize=18)

    plt.subplot(4,1,4)
    plt.plot(np.arange(q3,x1.shape[0]),x1[q3:x1.shape[0]],'g')
    plt.ylim(0, 10)
    plt.yticks(yticks)
    plt.xlabel('frames',fontsize=18)
    plt.ylabel('DF/F',fontsize=18)

    return 0

def compute_dff(traces, save_plot_dir=None, mode_kernelsize=5400, mean_kernelsize=3000):
    """ Compute dF/F of a set of traces using a mean-shifted windowed mode operator. 
    The operation is basically:  

        T_mm = windowed_mean(windowed_mode(T))

        T_dff = (T - T_mm) / T_mm
    
    Parameters
    ----------
    traces: np.ndarray
       2D array of traces to be analyzed

    Returns
    -------
    np.ndarray with the same shape as the input array.
    """

    if save_plot_dir is not None and not os.path.exists(save_plot_dir):
        os.makedirs(save_plot_dir)

    logging.debug("trace matrix shape: %d %d" % (traces.shape[0], traces.shape[1]))

    modeline = np.zeros(traces.shape[1])
    modelineLP = np.zeros(traces.shape[1])
    dff = np.zeros((traces.shape[0], traces.shape[1]))

    logging.debug("computing df/f")

    for n in range(0,traces.shape[0]):
        movingmode_fast(traces[n,:], mode_kernelsize, modeline[:])
        movingaverage(modeline[:], mean_kernelsize, modelineLP[:])
	dff[n,:] = (traces[n,:] - modelineLP[:]) / modelineLP[:]

        logging.debug("finished trace %d/%d" % (n+1, traces.shape[0]))

        if save_plot_dir:
            fig = plt.figure(figsize=(150,40))
            plot_onetrace(dff[n,:])

            plt.title('ROI '+str(n)+' ', fontsize=18)
            fig.savefig(os.path.join(save_plot_dir,'dff_%d.png' % n), orientation='landscape')
            plt.close(fig)

    return dff

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("input_h5")
    parser.add_argument("output_h5")
    parser.add_argument("--plot_dir")
    parser.add_argument("--log_level", default=logging.INFO)

    args = parser.parse_args()

    logging.getLogger().setLevel(args.log_level)

    # read from "data"
    if args.input_h5.endswith("nwb"):
        timestamps, traces = BrainObservatoryNwbDataSet(args.input_h5).get_corrected_fluorescence_traces()
    else:
        input_h5 = h5py.File(args.input_h5, "r")
        traces = input_h5["data"].value
        input_h5.close()

    dff = compute_dff(traces, args.plot_dir)
    
    # write to "data"
    output_h5 = h5py.File(args.output_h5, "w")
    output_h5["data"] = dff
    output_h5.close()

if __name__ == "__main__": main()



import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt

from allensdk.api.queries.glif_api import GlifApi
import allensdk.core.json_utilities as json_utilities
from allensdk.model.glif.glif_neuron import GlifNeuron
from allensdk.model.glif.simulate_neuron import simulate_neuron
import os, shutil, logging

#NEURONAL_MODEL_ID = 491547163 # level 1 LIF
NEURONAL_MODEL_ID = 491547171 # level 5 GLIF

OUTPUT_DIR = 'tmp'

def test_download():
    if os.path.exists(OUTPUT_DIR):
        shutil.rmtree(OUTPUT_DIR)
        
    os.makedirs(OUTPUT_DIR)

    glif_api = GlifApi()
    glif_api.get_neuronal_model(NEURONAL_MODEL_ID)
    glif_api.cache_stimulus_file(os.path.join(OUTPUT_DIR, '%d.nwb' % NEURONAL_MODEL_ID))

    neuron_config = glif_api.get_neuron_config()
    json_utilities.write(os.path.join(OUTPUT_DIR, '%d_neuron_config.json' % NEURONAL_MODEL_ID), neuron_config)

    ephys_sweeps = glif_api.get_ephys_sweeps()
    json_utilities.write(os.path.join(OUTPUT_DIR, 'ephys_sweeps.json'), ephys_sweeps)

def test_run():
    import allensdk.core.json_utilities as json_utilities


    # initialize the neuron
    neuron_config = json_utilities.read(os.path.join(OUTPUT_DIR, '%d_neuron_config.json' % NEURONAL_MODEL_ID))
    neuron = GlifNeuron.from_dict(neuron_config)

    # make a short square pulse. stimulus units should be in Amps.
    stimulus = [ 0.0 ] * 100 + [ 10e-9 ] * 100 + [ 0.0 ] * 100

    # important! set the neuron's dt value for your stimulus in seconds
    neuron.dt = 5e-6

    # simulate the neuron
    output = neuron.run(stimulus)

    voltage = output['voltage']
    threshold = output['threshold']

    plt.plot(voltage)
    plt.plot(threshold)
    plt.savefig(os.path.join(OUTPUT_DIR, 'plot.png'))

def test_simulate():
    logging.getLogger().setLevel(logging.DEBUG)
    neuron_config = json_utilities.read(os.path.join(OUTPUT_DIR, '%d_neuron_config.json' % NEURONAL_MODEL_ID))
    ephys_sweeps = json_utilities.read(os.path.join(OUTPUT_DIR, 'ephys_sweeps.json'))
    ephys_file_name = os.path.join(OUTPUT_DIR, '%d.nwb' % NEURONAL_MODEL_ID)

    neuron = GlifNeuron.from_dict(neuron_config)

    sweep_numbers = [ s['sweep_number'] for s in ephys_sweeps ]
    simulate_neuron(neuron, sweep_numbers, ephys_file_name, ephys_file_name, 0.05)

if __name__ == "__main__": 
    #test_download()
    #test_run()
    test_simulate()



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

from allensdk.api.queries.rma_template import RmaTemplate

class OntologiesApi(RmaTemplate):
    '''
    See: `Atlas Drawings and Ontologies
    <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies>`_
    '''
    
    rma_templates = \
        {"ontology_queries": [
            {'name': 'structures_by_graph_ids',
             'description': 'see name',
             'model': 'Structure',
             'criteria': '[graph_id$in{{ graph_ids }}]',
             'order': ['structures.graph_order'],
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['graph_ids']
            },
            {'name': 'structures_by_graph_names',
             'description': 'see name',
             'model': 'Structure',
             'criteria': 'graph[structure_graphs.name$in{{ graph_names }}]',
             'order': ['structures.graph_order'],
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['graph_names']
            },
            {'name': 'structures_by_set_ids',
             'description': 'see name',
             'model': 'Structure',
             'criteria': '[structure_set_id$in{{ set_ids }}]',
             'order': ['structures.graph_order'],
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['set_ids']
            },
            {'name': 'structures_by_set_names',
             'description': 'see name',
             'model': 'Structure',
             'criteria': 'structure_sets[name$in{{ set_names }}]',
             'order': ['structures.graph_order'],
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['set_names']
            },
            {'name': 'structure_graphs_list',
             'description': 'see name',
             'model': 'StructureGraph',
             'num_rows': 'all',
             'count': False
            },
            {'name': 'structure_sets_list',
             'description': 'see name',
             'model': 'StructureSet',
             'num_rows': 'all',
             'count': False
            },
            {'name': 'atlases_list',
             'description': 'see name',
             'model': 'Atlas',
             'num_rows': 'all',
             'count': False
            },
            {'name': 'atlases_table',
             'description': 'see name',
             'model': 'Atlas',
             'criteria': '{% if atlas_ids is defined %}[id$in{{ atlas_ids }}],{%endif%}structure_graph(ontology),graphic_group_labels',
             'include': 'structure_graph(ontology),graphic_group_labels',
             'only': ['atlases.id',
                      'atlases.name',
                      'atlases.image_type',
                      'ontologies.id',
                      'ontologies.name',
                      'structure_graphs.id',
                      'structure_graphs.name',
                      'graphic_group_labels.id',
                      'graphic_group_labels.name'],
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['atlas_ids']
            }
        ]}
    
    
    def __init__(self, base_uri=None):
        super(OntologiesApi, self).__init__(base_uri,
                                            query_manifest=OntologiesApi.rma_templates)
    
    
    def get_structures(self,
                       structure_graph_ids=None,
                       structure_graph_names=None,
                       structure_set_ids=None,
                       structure_set_names=None,
                       order = ['structures.graph_order'],
                       num_rows='all',
                       count=False):
        '''Retrieve data about anatomical structures.
        
        Parameters
        ----------
        structure_graph_ids : int or list of ints, optional
            database keys to get all structures in particular graphs
        structure_graph_names : string or list of strings, optional
            list of graph names to narrow the query
        structure_set_ids : int or list of ints, optional
            database keys to get all structures in a particular set
        structure_set_names : string or list of strings, optional
            list of set names to narrow the query.
        order : list of strings
            list of RMA order clauses for sorting
        num_rows : int
            how many records to retrieve
            
        Returns
        -------
        dict
            the parsed json response containing data from the API
            
        Notes
        -----
        Only one of the methods of limiting the query should be used at a time.
        '''
        if structure_graph_ids != None:
            data = self.template_query('ontology_queries',
                                       'structures_by_graph_ids',
                                       graph_ids=structure_graph_ids,
                                       order=order,
                                       num_rows=num_rows,
                                       count=count)
        elif structure_graph_names != None:
            data = self.template_query('ontology_queries',
                                       'structures_by_graph_names',
                                       graph_names=structure_graph_names,
                                       order=order,
                                       num_rows=num_rows,
                                       count=count)
        elif structure_set_ids != None:
            data = self.template_query('ontology_queries',
                                       'structures_by_set_ids',
                                       set_ids=structure_set_ids,
                                       order=order,
                                       num_rows=num_rows,
                                       count=count)
        elif structure_set_names != None:
            data = self.template_query('ontology_queries',
                                       'structures_by_set_names',
                                       set_names=structure_set_names,
                                       order=order,
                                       num_rows=num_rows,
                                       count=count)
        
        return data
    
    
    def unpack_structure_set_ancestors(self, structure_dataframe):
        '''Convert a slash-separated structure_id_path field to a list.
        
        Parameters
        ----------
        structure_dataframe : DataFrame
            structure data from the API
        
        Returns
        -------
        None
            A new column is added to the dataframe containing the ancestor list.
        '''
        ancestors = structure_dataframe['structure_id_path'].apply(
            lambda e: [int(a) for a in e.split('/')[1:-1]])
        structure_ancestors = [
            [n for n in ancestors_n] for ancestors_n in ancestors
        ]
        structure_dataframe['structure_set_ancestor'] = structure_ancestors


    def get_atlases_table(self, atlas_ids=None, brief=True):
        '''List Atlases available through the API
        with associated ontologies and structure graphs.
        
        Parameters
        ----------
        atlas_ids : integer or list of integers, optional
            only select specific atlases
        brief : boolean, optional
            True (default) requests only name and id fields.
        
        Returns
        -------
        dict : atlas metadata
        
        Notes
        -----
        This query is based on the
        `table of available Atlases <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies>`_.
        See also: `Class: Atlas <http://api.brain-map.org/doc/Atlas.html>`_
        '''
        if brief == True:
            data = self.template_query('ontology_queries',
                                       'atlases_table',
                                       atlas_ids=atlas_ids)
        else:
            data = self.template_query('ontology_queries',
                                       'atlases_table',
                                       atlas_ids=atlas_ids,
                                       only=None)
        
        return data
    
    
    def get_atlases(self):
        return self.template_query('ontology_queries',
                                   'atlases_list')
    
    
    def get_structure_graphs(self):
        return self.template_query('ontology_queries',
                                   'structure_graphs_list')
        
    def get_structure_sets(self):
        return self.template_query('ontology_queries',
                                   'structure_sets_list')



import pytest
from mock import MagicMock
from allensdk.api.api import Api
import allensdk.core.json_utilities as ju


@pytest.fixture
def api():
    ju.read_url_post = \
        MagicMock(name='read_url_post',
                  return_value={ 'whatever': True })

    api = Api()
    
    return api
    

def test_do_query_post(api):
    api.do_query(lambda *a, **k: 'http://localhost/%s' % (a[0]),
                 lambda d: d,
                 "wow",
                 post=True)
    
    ju.read_url_post.assert_called_once_with('http://localhost/wow')


def test_do_query_get(api):
    ju.read_url_get = \
        MagicMock(name='read_url_get',
                  return_value={ 'whatever': True })
            
    api.do_query(lambda *a, **k: 'http://localhost/%s' % (a[0]),
                 lambda d: d,
                 "wow",
                 post=False)
     
    ju.read_url_get.assert_called_once_with('http://localhost/wow')
 
 
def test_load_api_schema(api):
    ju.read_url_get = \
        MagicMock(name='read_url_get',
                  return_value={ 'whatever': True })
            
    api.load_api_schema()
     
    ju.read_url_get.assert_called_once_with(
        'http://api.brain-map.org/api/v2/data/enumerate.json')



from allensdk.api.queries.ontologies_api import OntologiesApi

import pytest
from mock import MagicMock


@pytest.fixture
def ontologies():
    oa = OntologiesApi()
    oa.json_msg_query = MagicMock(name='json_msg_query')
    
    return oa


def test_get_structure_graph(ontologies):
    structure_graph_id = 1
    ontologies.get_structures(structure_graph_id)
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,[graph_id$in1],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_list_structure_graphs(ontologies):
    ontologies.get_structure_graphs()
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::StructureGraph,"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_list_structure_sets(ontologies):
    ontologies.get_structure_sets()
    
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::StructureSet,rma::options[num_rows$eq'all'][count$eqfalse]")


def test_list_atlases(ontologies):
    ontologies.get_atlases()
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Atlas,rma::options[num_rows$eq'all'][count$eqfalse]")


def test_structure_graph_by_name(ontologies):
    ontologies.get_structures(structure_graph_names="'Mouse Brain Atlas'")
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,"
        "graph[structure_graphs.name$in'Mouse Brain Atlas'],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_graphs_by_names(ontologies):
    ontologies.get_structures(structure_graph_names=["'Mouse Brain Atlas'",
                                                     "'Human Brain Atlas'"])
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,"
        "graph[structure_graphs.name$in'Mouse Brain Atlas',"
        "'Human Brain Atlas'],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_set_by_id(ontologies):
    ontologies.get_structures(structure_set_ids=8)
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,[structure_set_id$in8],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_sets_by_ids(ontologies):
    ontologies.get_structures(structure_set_ids=[7,8])
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,[structure_set_id$in7,8],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_set_by_name(ontologies):
    ontologies.get_structures(
        structure_set_names=ontologies.quote_string(
            "Mouse Connectivity - Summary"))
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,"
        "structure_sets[name$in'Mouse Connectivity - Summary'],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_set_by_names(ontologies):
    ontologies.get_structures(
        structure_set_names=[
            ontologies.quote_string("NHP - Coarse"),
            ontologies.quote_string("Mouse Connectivity - Summary")])
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,"
        "structure_sets[name$in'NHP - Coarse','Mouse Connectivity - Summary'],"
        "rma::options"
        "[num_rows$eq'all'][order$eqstructures.graph_order][count$eqfalse]")


def test_structure_set_no_order(ontologies):
    ontologies.get_structures(1, order=None)
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Structure,rma::criteria,"
        "[graph_id$in1],rma::options[num_rows$eq'all'][count$eqfalse]")


def test_atlas_1(ontologies):
    atlas_id = 1
    ontologies.get_atlases_table(atlas_id)
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Atlas,rma::criteria,"
        "[id$in1],structure_graph(ontology),graphic_group_labels,"
        "rma::include,structure_graph(ontology),graphic_group_labels,"
        "rma::options[only$eq'atlases.id,atlases.name,atlases.image_type,"
        "ontologies.id,ontologies.name,"
        "structure_graphs.id,structure_graphs.name,"
        "graphic_group_labels.id,graphic_group_labels.name']"
        "[num_rows$eq'all'][count$eqfalse]")


def test_atlas_verbose(ontologies):
    ontologies.get_atlases_table(brief=False)
    ontologies.json_msg_query.assert_called_once_with(
        "http://api.brain-map.org/api/v2/data/query.json?q="
        "model::Atlas,rma::criteria,"
        "structure_graph(ontology),graphic_group_labels,"
        "rma::include,structure_graph(ontology),graphic_group_labels,"
        "rma::options[num_rows$eq'all'][count$eqfalse]")



# Copyright 2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import os
import allensdk.core.json_utilities as ju
from allensdk.api.cache import Cache
from allensdk.api.queries.brain_observatory_api import BrainObservatoryApi
from allensdk.config.manifest_builder import ManifestBuilder
from allensdk.core.brain_observatory_nwb_data_set import BrainObservatoryNwbDataSet
import allensdk.brain_observatory.stimulus_info as stim_info
import pandas as pd


class BrainObservatoryCache(Cache):
    """
    Cache class for storing and accessing data from the Brain Observatory.
    By default, this class will cache any downloaded metadata or files in 
    well known locations defined in a manifest file.  This behavior can be
    disabled.

    Attributes
    ----------
    
    api: BrainObservatoryApi instance
        The object used for making API queries related to the Brain

 Observatory.

    Parameters
    ----------
    
    cache: boolean
        Whether the class should save results of API queries to locations specified
        in the manifest file.  Queries for files (as opposed to metadata) must have a
        file location.  If caching is disabled, those locations must be specified
        in the function call (e.g. get_ophys_experiment_data(file_name='file.nwb')).

    manifest_file: string
       File name of the manifest to be read.  Default is "brain_observatory_manifest.json".
    """

    EXPERIMENT_CONTAINERS_KEY = 'EXPERIMENT_CONTAINERS'
    EXPERIMENTS_KEY = 'EXPERIMENTS'
    CELL_SPECIMENS_KEY = 'CELL_SPECIMENS'
    EXPERIMENT_DATA_KEY = 'EXPERIMENT_DATA'
    STIMULUS_MAPPINGS_KEY = 'STIMULUS_MAPPINGS'
    
    def __init__(self, cache=True, manifest_file='brain_observatory_manifest.json', base_uri=None):
        super(BrainObservatoryCache, self).__init__(manifest=manifest_file, cache=cache)
        self.api = BrainObservatoryApi(base_uri=base_uri)
        

    def get_all_targeted_structures(self):
        """ Return a list of all targeted structures in the data set. """
        containers = self.get_experiment_containers(simple=False)
        targeted_structures = set([ c['targeted_structure']['acronym'] for c in containers])
        return sorted(list(targeted_structures))


    def get_all_cre_lines(self):
        """ Return a list of all cre driver lines in the data set. """
        containers = self.get_experiment_containers(simple=False)
        cre_lines = set([ _find_specimen_cre_line(c['specimen']) for c in containers ])
        return sorted(list(cre_lines))


    def get_all_imaging_depths(self):
        """ Return a list of all imaging depths in the data set. """
        containers = self.get_experiment_containers(simple=False)
        imaging_depths = set([ c['imaging_depth'] for c in containers ])
        return sorted(list(imaging_depths))


    def get_all_session_types(self):
        """ Return a list of all stimulus sessions in the data set. """
        exps = self.get_ophys_experiments()
        names = set([ exp['stimulus_name'] for exp in exps ])
        return sorted(list(names))


    def get_all_stimuli(self):
        """ Return a list of all stimuli in the data set. """
        return sorted(list(stim_info.all_stimuli()))
    

    def get_experiment_containers(self, file_name=None, 
                                  ids=None, 
                                  targeted_structures=None, 
                                  imaging_depths=None, 
                                  cre_lines=None, 
                                  simple=True):
        """ Get a list of experiment containers matching certain criteria.
        
        Parameters
        ----------
        file_name: string
            File name to save/read the experiment containers.  If file_name is None, 
            the file_name will be pulled out of the manifest.  If caching
            is disabled, no file will be saved. Default is None.

        ids: list
            List of experiment container ids.  

        targeted_structures: list
            List of structure acronyms.  Must be in the list returned by 
            BrainObservatoryCache.get_all_targeted_structures().

        imaging_depths: list
            List of imaging depths.  Must be in the list returned by 
            BrainObservatoryCache.get_all_imaging_depths().

        cre_lines: list
            List of cre lines.  Must be in the list returned by 
            BrainObservatoryCache.get_all_cre_lines().

        simple: boolean
            Whether or not to simplify the dictionary properties returned by this method
            to a more concise subset.

        Returns
        -------
        list of dictionaries
        """
        file_name = self.get_cache_path(file_name, self.EXPERIMENT_CONTAINERS_KEY)

        if os.path.exists(file_name):
            containers = ju.read(file_name)
        else:
            containers = self.api.get_experiment_containers()

            if self.cache:
                ju.write(file_name, containers)

        containers = self.api.filter_experiment_containers(containers, ids=ids,
                                                           targeted_structures=targeted_structures, 
                                                           imaging_depths=imaging_depths, 
                                                           transgenic_lines=cre_lines)

        if simple:
            containers = [ {
                    'id': c['id'],
                    'imaging_depth': c['imaging_depth'],
                    'targeted_structure': c['targeted_structure']['acronym'],
                    'cre_line': _find_specimen_cre_line(c['specimen']),
                    'age_days': c['specimen']['donor']['age']['days']
                    } for c in containers ]

        return containers


    def get_ophys_experiments(self, file_name=None, 
                              ids=None,
                              experiment_container_ids=None,
                              targeted_structures=None, 
                              imaging_depths=None, 
                              cre_lines=None,
                              stimuli=None, 
                              session_types=None, 
                              simple=True):
        """ Get a list of ophys experiments matching certain criteria.
        
        Parameters
        ----------
        file_name: string
            File name to save/read the ophys experiments.  If file_name is None, 
            the file_name will be pulled out of the manifest.  If caching
            is disabled, no file will be saved. Default is None.

        ids: list
            List of ophys experiment ids.  

        experiment_container_ids: list
            List of experiment container ids.

        targeted_structures: list
            List of structure acronyms.  Must be in the list returned by 
            BrainObservatoryCache.get_all_targeted_structures().

        imaging_depths: list
            List of imaging depths.  Must be in the list returned by 
            BrainObservatoryCache.get_all_imaging_depths().

        cre_lines: list
            List of cre lines.  Must be in the list returned by 
            BrainObservatoryCache.get_all_cre_lines().

        stimuli: list
            List of stimulus names.  Must be in the list returned by 
            BrainObservatoryCache.get_all_stimuli().

        session_types: list
            List of stimulus session type names.  Must be in the list returned by 
            BrainObservatoryCache.get_all_session_types().

        simple: boolean
            Whether or not to simplify the dictionary properties returned by this method
            to a more concise subset.

        Returns
        -------
        list of dictionaries
        """
        file_name = self.get_cache_path(file_name, self.EXPERIMENTS_KEY)

        if os.path.exists(file_name):
            exps = ju.read(file_name)
        else:
            exps = self.api.get_ophys_experiments()

            if self.cache:
                ju.write(file_name, exps)

        exps = self.api.filter_ophys_experiments(exps, 
                                                 ids=ids,
                                                 experiment_container_ids=experiment_container_ids, 
                                                 targeted_structures=targeted_structures, 
                                                 imaging_depths=imaging_depths, 
                                                 transgenic_lines=cre_lines, 
                                                 stimuli=stimuli, 
                                                 session_types=session_types)

        if simple:
            exps = [ {
                    'id': e['id'],
                    'imaging_depth': e['imaging_depth'],
                    'targeted_structure': e['targeted_structure']['acronym'],
                    'cre_line': _find_specimen_cre_line(e['specimen']),
                    'age_days': e['specimen']['donor']['age']['days'],
                    'experiment_container_id': e['experiment_container_id'],
                    'session_type': e['stimulus_name']
                    } for e in exps ]
        return exps


    def _get_stimulus_mappings(self, file_name=None):
        """ Returns a mapping of which metrics are related to which stimuli. Internal use only. """

        file_name = self.get_cache_path(file_name, self.STIMULUS_MAPPINGS_KEY)

        if os.path.exists(file_name):
            mappings = ju.read(file_name)
        else:
            mappings = self.api.get_stimulus_mappings()

            if self.cache:
                ju.write(file_name, mappings)

        return mappings


    def get_cell_specimens(self, file_name=None, ids=None, experiment_container_ids=None, simple=True):
        """ Return cell specimens that have certain properies.
        
        Parameters
        ----------
        file_name: string
            File name to save/read the cell specimens.  If file_name is None, 
            the file_name will be pulled out of the manifest.  If caching
            is disabled, no file will be saved. Default is None.

        ids: list
            List of cell specimen ids.  

        experiment_container_ids: list
            List of experiment container ids.
            
        simple: boolean
            Whether or not to simplify the dictionary properties returned by this method
            to a more concise subset.
            
        Returns
        -------
        list of dictionaries
        """

        file_name = self.get_cache_path(file_name, self.CELL_SPECIMENS_KEY)

        if os.path.exists(file_name):
            cell_specimens = ju.read(file_name)
        else:
            cell_specimens = self.api.get_cell_metrics()

            if self.cache:
                ju.write(file_name, cell_specimens)

        cell_specimens = self.api.filter_cell_specimens(cell_specimens, 
                                                        ids=ids, 
                                                        experiment_container_ids=experiment_container_ids)
        
        # drop the thumbnail columns
        if simple:
            mappings = self._get_stimulus_mappings()
            thumbnails = [ m['item'] for m in mappings if m['item_type'] == 'T' and m['level'] == 'R']
            for cs in cell_specimens:
                for t in thumbnails:
                    del cs[t]
                
        return cell_specimens

    
    def get_ophys_experiment_data(self, ophys_experiment_id, file_name=None):
        """ Download the NWB file for an ophys_experiment (if it hasn't already been
        downloaded) and return a data accessor object.

        Parameters
        ----------
        file_name: string
            File name to save/read the data set.  If file_name is None, 
            the file_name will be pulled out of the manifest.  If caching
            is disabled, no file will be saved. Default is None.        

        ophys_experiment_id: integer
            id of the ophys_experiment to retrieve
            
        Returns
        -------
        BrainObservatoryNwbDataSet
        """
        file_name = self.get_cache_path(file_name, self.EXPERIMENT_DATA_KEY, ophys_experiment_id)

        if not os.path.exists(file_name):
            self.api.save_ophys_experiment_data(ophys_experiment_id, file_name)

        return BrainObservatoryNwbDataSet(file_name)

    
    def build_manifest(self, file_name):
        """
        Construct a manifest for this Cache class and save it in a file.
        
        Parameters
        ----------
        
        file_name: string
            File location to save the manifest.

        """

        mb = ManifestBuilder()

        mb.add_path('BASEDIR', '.')
        mb.add_path(self.EXPERIMENT_CONTAINERS_KEY, 'experiment_containers.json', typename='file', parent_key='BASEDIR')
        mb.add_path(self.EXPERIMENTS_KEY, 'ophys_experiments.json', typename='file', parent_key='BASEDIR')
        mb.add_path(self.EXPERIMENT_DATA_KEY, 'ophys_experiment_data/%d.nwb', typename='file', parent_key='BASEDIR')
        mb.add_path(self.CELL_SPECIMENS_KEY, 'cell_specimens.json', typename='file', parent_key='BASEDIR')
        mb.add_path(self.STIMULUS_MAPPINGS_KEY, 'stimulus_mappings.json', typename='file', parent_key='BASEDIR')

        mb.write_json_file(file_name)

def _find_specimen_cre_line(specimen):
    return next(tl['name'] for tl in specimen['donor']['transgenic_lines'] if 'Cre' in tl['name'])



# Copyright 2015 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

from allensdk.api.queries.rma_api import RmaApi
from jinja2 import Template

class RmaTemplate(RmaApi):
    '''
    See: `Atlas Drawings and Ontologies
    <http://help.brain-map.org/display/api/Atlas+Drawings+and+Ontologies>`_
    '''
    
    def __init__(self, base_uri=None, query_manifest=None):
        super(RmaTemplate, self).__init__(base_uri)
        self.templates = query_manifest
    
    
    def to_filter_rhs(self, rhs):
        if type(rhs) == list:
            return ','.join(str(r) for r in rhs)
        
        return rhs
    
    
    def template_query(self, template_name, entry_name, **kwargs):
        cb = self.templates[template_name]
        templates = [e for e in cb if e['name'] == entry_name]
        
        if len(templates) > 0:
            template = templates[0]
        else:
            raise Exception('Entry %s not found.' % (entry_name))
        
        query_args = {'model': template['model']}
        
        if 'criteria' in template:
            criteria_template = Template(template['criteria'])
            
            if 'criteria_params' in template:
                criteria_params = { key: self.to_filter_rhs(kwargs.get(key))
                                    for key in template['criteria_params']
                                    if key in kwargs and kwargs.get(key) is not None }
            else:
                criteria_params = {}
       
            criteria_str = str(criteria_template.render(**criteria_params))
            if criteria_str:
                query_args['criteria'] = criteria_str
                

        
        if 'include' in template:
            include_template = Template(template['include'])
            
            if 'include_params' in template:
                include_params = { key: self.to_filter_rhs(kwargs.get(key))
                                   for key in template['include_params']
                                   if key in kwargs and kwargs.get(key) is not None}
            else:
                include_params = {}
            
            include_str = str(include_template.render(**include_params))
            if include_str:
                query_args['include'] = include_str
                
        
        if 'only' in kwargs:
            if kwargs.get('only') != None:
                query_args['only'] = [self.quote_string(','.join(kwargs.get('only')))]
        elif 'only' in template:
            query_args['only'] = [self.quote_string(','.join(template['only']))]
        
        if 'except' in kwargs:
            if kwargs.get('except') != None:
                query_args['except'] = [self.quote_string(','.join(kwargs.get('except')))]
        elif 'except' in template:
            query_args['except'] = template['except']
        
        if 'start_row' in kwargs:
            query_args['start_row'] = kwargs.get('start_row')
        elif 'start_row' in template:
            query_args['start_row'] = template['start_row']
        
        if 'num_rows' in kwargs:
            query_args['num_rows'] = kwargs.get('num_rows')
        elif 'num_rows' in template:
            query_args['num_rows'] = template['num_rows']
        
        if 'count' in kwargs:
            query_args['count'] = kwargs.get('count')
        elif 'count' in template:
            query_args['count'] = template['count']
        
        if 'order' in kwargs:
            query_args['order'] = kwargs.get('order')
        elif 'order' in template:
            query_args['order'] = template['order']

        data = self.model_query(**query_args)

        return data




# Copyright 2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.

import matplotlib.pyplot as plt
from matplotlib.ticker import MaxNLocator
import numpy as np
import pandas as pd
import os, logging
from numpy import arange

def plot_drifting_grating_traces(dg, save_dir):
    '''saves figures with a Ori X TF grid of mean resposes'''
    logging.info("Plotting Ori and TF mean response for all cells")
    
    blank =  dg.sweep_response[dg.stim_table.temporal_frequency==0]          
    for nc in range(dg.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
        xtime = np.arange(-1*dg.interlength/dg.acquisition_rate, (dg.sweeplength+dg.interlength)/dg.acquisition_rate, 1/dg.acquisition_rate)
        plt.figure(nc, figsize=(20,16))
        vmax=0
        vmin=0
        try:
            blank_p =  blank[str(nc)].mean() + (blank[str(nc)].std()/len(blank[str(nc)]))
            blank_n =  blank[str(nc)].mean() - (blank[str(nc)].std()/len(blank[str(nc)]))
        except:
            blank_p = blank.iloc[:,nc].apply(np.mean) + (blank.iloc[:,nc].apply(np.std) / blank.iloc[:,nc].apply(len))
            blank_n = blank.iloc[:,nc].apply(np.mean) - (blank.iloc[:,nc].apply(np.std) / blank.iloc[:,nc].apply(len))
        for ori in dg.orivals:
            ori_pt = np.where(dg.orivals == ori)[0][0]
            for tf in dg.tfvals[1:]:
                tf_pt = np.where(dg.tfvals == tf)[0][0]
                sp_pt = (5*ori_pt)+tf_pt
                subset_response = dg.sweep_response[(dg.stim_table.temporal_frequency==tf)&(dg.stim_table.orientation==ori)]               
                try:
                    subset_response_p =  subset_response[str(nc)].mean() + (subset_response[str(nc)][:-1].std()/len(subset_response[str(nc)]))
                    subset_response_n =  subset_response[str(nc)].mean() - (subset_response[str(nc)][:-1].std()/len(subset_response[str(nc)]))
                except:
                    subset_response_p =  subset_response.iloc[:,nc].apply(np.mean) + (subset_response.iloc[:,nc].apply(np.std) / subset_response.iloc[:,nc].apply(len))
                    subset_response_n =  subset_response.iloc[:,nc].apply(np.mean) - (subset_response.iloc[:,nc].apply(np.std) / subset_response.iloc[:,nc].apply(len))
                ax = plt.subplot(8,5,sp_pt)
                while len(xtime)>len(subset_response[str(nc)].mean()):
                    xtime = np.delete(xtime, -1)
                try:
                    ax.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
                except:
                    pass
                try:
                    ax.fill_between(xtime, blank_p, blank_n, color='k', alpha=0.5)
                except:
                    pass
                try:
                    ax.plot(xtime, subset_response[str(nc)].mean(), color='b', lw=2)
                except:
                    pass
                ax.plot(xtime, subset_response[str(nc)].mean(), color='b', lw=2)
                #TODO: remove the [:119]  and [:-1] and the try/except                 
                ax.plot(xtime, blank[str(nc)].mean(), color='k', lw=2)
                ax.axvspan(0, dg.sweeplength/dg.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
                ax.set_xlim(-1,3)
                ax.set_xticks(range(-1,4))
                ax.yaxis.set_major_locator(MaxNLocator(4))                 
                vmax = np.where(np.amax(subset_response_p)>vmax, np.amax(subset_response_p), vmax)
                vmin = np.where(np.amin(subset_response_n)<vmin, np.amin(subset_response_n), vmin)                 

                if ori_pt<7:
                    ax.set_xticks([])
                else:
                    ax.set_xlabel("Time (s)", fontsize=20)
                if tf_pt>1:
                    ax.set_yticks([])
                else:
                    ax.set_ylabel(str(dg.orivals[ori_pt]), fontsize=24)
                if ori_pt==0:
                    ax.set_title(str(dg.tfvals[tf_pt]), fontsize=24)
                
        for i in range(1,sp_pt+1):
            ax = plt.subplot(8,5,i)
            ax.set_ylim(vmin, vmax)
        plt.tick_params(labelsize=16)
        plt.tight_layout()
        plt.suptitle("Cell " + str(nc+1), fontsize=20)
        plt.subplots_adjust(top=0.9)
        filename = 'Traces DG Cell_'+str(nc+1)+'.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()

def plot_ns_traces(nsa, save_dir):
    logging.info("Plotting Natural Scene traces for each cell")
    xtime = np.arange(-1*nsa.interlength/nsa.acquisition_rate, (nsa.sweeplength+nsa.interlength)/nsa.acquisition_rate, 1/nsa.acquisition_rate)
    blank = nsa.sweep_response[nsa.stim_table.frame==-1]       
    for nc in range(nsa.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
        vmax=0
        vmin=0
        blank_p =  blank[str(nc)].mean() + (blank[str(nc)].std()/len(blank[str(nc)]))
        blank_n =  blank[str(nc)].mean() - (blank[str(nc)].std()/len(blank[str(nc)])) 
        plt.figure(nc, figsize=(30,25))
        for ns in range(nsa.number_scenes-1):
            subset_response = nsa.sweep_response[nsa.stim_table.frame==ns]               
            subset_response_p =  subset_response[str(nc)].mean() + (subset_response[str(nc)][:].std()/len(subset_response[str(nc)]))
            subset_response_n =  subset_response[str(nc)].mean() - (subset_response[str(nc)][:].std()/len(subset_response[str(nc)]))
            ax = plt.subplot(10,12,ns+1)
            try:
                ax.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
            except:
                xtime = xtime[:-1]
                ax.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
            ax.fill_between(xtime, blank_p, blank_n, color='k', alpha=0.5)                 
            ax.plot(xtime, subset_response[str(nc)].mean(), color='b', lw=2)                 
            ax.plot(xtime, blank[str(nc)].mean(), color='k', lw=2)
            ax.axvspan(0, nsa.sweeplength/nsa.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
            ax.yaxis.set_major_locator(MaxNLocator(4))                 
            vmax = np.where(np.amax(subset_response_p)>vmax, np.amax(subset_response_p), vmax)
            vmin = np.where(np.amin(subset_response_n)<vmin, np.amin(subset_response_n), vmin)     
            if ns<108:
                ax.set_xticks([])
            if np.mod(ns,12):
                ax.set_yticks([])
        for i in range(1,nsa.number_scenes):
            ax = plt.subplot(10,12,i)
            ax.set_ylim(vmin, vmax)
        plt.tight_layout()
        plt.suptitle("Cell " + str(nc+1), fontsize=20)
        plt.subplots_adjust(top=0.9)
        filename = 'NS Traces Cell_'+str(nc+1)+'.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()                

def plot_sg_traces(sg, save_dir):
    logging.info("Plotting Static Grating traces for each cell")
    xtime = np.arange(-1*sg.interlength/sg.acquisition_rate, (sg.sweeplength+sg.interlength)/sg.acquisition_rate, 1/sg.acquisition_rate)     
    blank = sg.sweep_response[sg.stim_table.spatial_frequency==0]
    for nc in range(sg.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
        vmax=0
        vmin=0
        blank_p =  blank[str(nc)].mean() + (blank[str(nc)].std()/len(blank[str(nc)]))
        blank_n =  blank[str(nc)].mean() - (blank[str(nc)].std()/len(blank[str(nc)]))
        while len(xtime)>len(blank_p):
            xtime = np.delete(xtime, -1)  
        plt.figure(nc, figsize=(30,30))
        ph_dict = {0:0, 0.25:6, 0.5:77, 0.75:83}
        for ori in sg.orivals:
            ori_pt = np.where(sg.orivals == ori)[0][0]
            for sf in sg.sfvals[1:]:
                sf_pt = np.where(sg.sfvals == sf)[0][0]
                for phase in sg.phasevals:
                    ph_pt = ph_dict[phase]
                    subplotnum = sf_pt + (ori_pt*11) + ph_pt
                    subset_response = sg.sweep_response[(sg.stim_table.spatial_frequency==sf)&(sg.stim_table.orientation==ori)&(sg.stim_table.phase==phase)]
                    subset_response_p =  subset_response[str(nc)].mean() + (subset_response[str(nc)][:].std()/len(subset_response[str(nc)]))
                    subset_response_n =  subset_response[str(nc)].mean() - (subset_response[str(nc)][:].std()/len(subset_response[str(nc)]))
                    ax = plt.subplot(13,11,subplotnum)
                    ax.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
                    ax.fill_between(xtime, blank_p, blank_n, color='k', alpha=0.5)                 
                    ax.plot(xtime, subset_response[str(nc)].mean(), color='b', lw=2)                 
                    ax.plot(xtime, blank[str(nc)].mean(), color='k', lw=2)
                    ax.axvspan(0, sg.sweeplength/sg.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
                    ax.yaxis.set_major_locator(MaxNLocator(4))                 
                    vmax = np.where(np.amax(subset_response_p)>vmax, np.amax(subset_response_p), vmax)
                    vmin = np.where(np.amin(subset_response_n)<vmin, np.amin(subset_response_n), vmin)    
                    if np.mod(subplotnum,11)!=1:
                        ax.set_yticks([])
                    else:
                        ax.set_ylabel(ori, fontsize=20)
                    if subplotnum<133:
                        ax.set_xticks([])
                    if subplotnum<12:
                        ax.set_title(sf, fontsize=20)
                    if subplotnum==3:
                        ax.set_title("Phase 0.0", fontsize=20)
                    if subplotnum==9:
                        ax.set_title("Phase 0.25", fontsize=20)
                    if subplotnum==80:
                        ax.set_title("Phase 0.5", fontsize=20)
                    if subplotnum==86:
                        ax.set_title("Phase 0.75", fontsize=20)
        for i in range(1,144):
            ax = plt.subplot(13,11,i)
            ax.set_ylim(vmin, vmax)
        plt.tight_layout()
        plt.suptitle("Cell " + str(nc+1), fontsize=20)
        plt.subplots_adjust(top=0.9)
        filename = 'SG Traces Cell_'+str(nc+1)+'.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()                

def plot_lsn_traces(lsn, save_dir):
    logging.info("Plotting LSN traces for all cells")
    xtime = np.arange(-lsn.interlength/lsn.acquisition_rate, 
                       (lsn.interlength + lsn.sweeplength)/lsn.acquisition_rate, 
                       1.0/lsn.acquisition_rate)

    for nc in range(lsn.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
            
        plt.figure(nc, figsize=(24,20))
        vmax=0
        vmin=0
        one_cell = lsn.sweep_response[str(nc)]
        
        for yp in range(16):
            for xp in range(28):
                sp_pt = (yp*28)+xp+1
                on_frame = np.where(lsn.LSN[:,yp,xp]==255)[0]
                off_frame = np.where(lsn.LSN[:,yp,xp]==0)[0]
                subset_on = one_cell[lsn.stim_table.frame.isin(on_frame)]
                subset_off = one_cell[lsn.stim_table.frame.isin(off_frame)]

                subset_on_mean = subset_on.mean()
                subset_off_mean = subset_off.mean()

                ax = plt.subplot(16,28,sp_pt)
                ax.plot(xtime, subset_on_mean, color='r', lw=2)
                ax.plot(xtime, subset_off_mean, color='b', lw=2)
                ax.axvspan(0, lsn.sweeplength/lsn.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
                vmax = np.where(np.amax(subset_on_mean)>vmax, np.amax(subset_on_mean), vmax)
                vmax = np.where(np.amax(subset_off_mean)>vmax, np.amax(subset_off_mean), vmax)
                vmin = np.where(np.amin(subset_on_mean)<vmin, np.amin(subset_on_mean), vmin)
                vmin = np.where(np.amin(subset_off_mean)<vmin, np.amin(subset_off_mean), vmin)
                ax.set_xticks([])
                ax.set_yticks([])
                
        for i in range(1,sp_pt+1):
            ax = plt.subplot(16,28,i)
            ax.set_ylim(vmin, vmax)
        
        plt.tight_layout()
        plt.suptitle("Cell " + str(nc+1), fontsize=20)
        plt.subplots_adjust(top=0.9)
        filename = 'Traces LSN Cell_'+str(nc+1)+'.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()          


def _plot_3sa(dg, nm1, nm3, save_dir):
    logging.info("Plotting for all cell")
    for nc in range(dg.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
        plt.figure(nc, figsize=(20,20))
        ax1 = plt.subplot2grid((6,6), (0,0), colspan=4) #full trace
        ax2 = plt.subplot2grid((6,6), (0,4))            #histogram of F
        ax3 = plt.subplot2grid((6,6), (1,0), colspan=4) #running speed
        ax4 = plt.subplot2grid((6,6), (1,4))
        ax5 = plt.subplot2grid((6,6), (1,5), sharex=ax4, sharey=ax4)
        ax6 = plt.subplot2grid((6,6), (2,0), colspan=2)
        ax7 = plt.subplot2grid((6,6), (2,2), colspan=2)
        ax8 = plt.subplot2grid((6,6), (2,4), colspan=2)
        ax9 = plt.subplot2grid((6,6), (3,0))
        ax10 = plt.subplot2grid((6,6), (4,0), colspan=3)
        ax11 = plt.subplot2grid((6,6), (5,0), colspan=3)
        ax12 = plt.subplot2grid((6,6), (4,3), colspan=3)
        ax13 = plt.subplot2grid((6,6), (5,3), colspan=3)

        xtime = np.arange(0,np.size(dg.celltraces,1),1.)
        xtime /= dg.acquisition_rate
        dif = np.ediff1d(dg.stim_table.start.values, to_begin=8000, to_end=8000)
        test = np.argwhere(dif>5000)
        ax1.plot(xtime, dg.celltraces[nc,:])        
        for i in range(len(test)-1): 
            ax1.axvspan(xmin=(dg.stim_table.start.iloc[test[i]].values/dg.acquisition_rate), xmax=(dg.stim_table.end.iloc[test[i+1]-1].values/dg.acquisition_rate), color='gray', alpha=0.3)
        ax1.axvspan(xmin=nm1.stim_table.start.min()/nm1.acquisition_rate, xmax=((nm1.stim_table.start.max()+nm1.sweeplength)/nm1.acquisition_rate), color='red', alpha=0.3)
        dif = np.ediff1d(nm3.stim_table.start.values, to_begin=8000, to_end=8000)
        test = np.argwhere(dif>5000)
        for i in range(len(test)-1): 
            ax1.axvspan(xmin=(nm3.stim_table.start.iloc[test[i]].values/nm3.acquisition_rate), xmax=((nm3.stim_table.end.iloc[test[i+1]-1].values+nm3.sweeplength)/nm3.acquisition_rate), color='blue', alpha=0.3)
        ax1.set_xlabel("Time (s)", fontsize=20)
        ax1.set_ylabel("Fluorescence", fontsize=20)
        
        ax2.hist(dg.celltraces[nc,:], bins=70)
        ax2.set_yscale('log')
        ax2.set_xlabel("Fluorescence", fontsize=20)
        ax2.set_ylabel("Count", fontsize=20)
        
        xtime = np.arange(0, np.size(dg.dxcm), 1.)
        xtime /= dg.acquisition_rate
        ax3.plot(xtime, dg.dxcm, color='k')
        ax3.set_xlabel("Time (s)", fontsize=20)
        ax3.set_xlabel("Speed (cm/s)", fontsize=20)
        
        smax = nm1.binned_cells_sp[nc,np.argmax(nm1.binned_cells_sp[nc,:,0]),0] + nm1.binned_cells_sp[nc,np.argmax(nm1.binned_cells_sp[nc,:,0]),1]
        vmax = nm1.binned_cells_vis[nc,np.argmax(nm1.binned_cells_vis[nc,:,0]),0] + nm1.binned_cells_vis[nc,np.argmax(nm1.binned_cells_vis[nc,:,0]),1]
        rmax = np.where(smax>vmax, smax, vmax)
        smin = nm1.binned_cells_sp[nc,np.argmin(nm1.binned_cells_sp[nc,:,0]),0] - nm1.binned_cells_sp[nc,np.argmin(nm1.binned_cells_sp[nc,:,0]),1]
        vmin = nm1.binned_cells_vis[nc,np.argmin(nm1.binned_cells_vis[nc,:,0]),0] - nm1.binned_cells_vis[nc,np.argmin(nm1.binned_cells_vis[nc,:,0]),1]
        rmin = np.where(smin<vmin, smin, vmin)      
        
        ax4.errorbar(nm1.binned_dx_sp[:,0], nm1.binned_cells_sp[nc,:,0], yerr=nm1.binned_cells_sp[nc,:,1], fmt='.', color='k')
        ax4.set_ylim(rmin, rmax)        
        ax4.set_xlabel("Speed (cm/s)", fontsize=20)
        ax4.set_ylabel("DF/F", fontsize=20)
        ax4.set_title("Spontaneous", fontsize=20)
        
        ax5.errorbar(nm1.binned_dx_vis[:,0], nm1.binned_cells_vis[nc,:,0], yerr=nm1.binned_cells_vis[nc,:,1], fmt='.')
        ax5.set_ylim(rmin, rmax)        
        ax5.set_xlabel("Speed (cm/s)", fontsize=20)
        ax5.set_ylabel("DF/F", fontsize=20)
        ax5.set_title("Visual Stimuli", fontsize=20)
        
        peakori = dg.peak.ori_dg[nc]
        peaktf = dg.peak.tf_dg[nc]
        ax6.errorbar(dg.orivals, dg.response[:,peaktf,nc,0], yerr=dg.response[:,peaktf,nc,1], fmt='bo-', lw=2)
        ax6.fill_between(dg.orivals, np.repeat(dg.response[0,0,nc,0]+dg.response[0,0,nc,1], dg.number_ori), np.repeat(dg.response[0,0,nc,0]-dg.response[0,0,nc,1], dg.number_ori), color='gray', alpha=0.5)
        ax6.axhline(y=dg.response[0,0,nc,0], ls='--', color='k')
        ax6.annotate(str(dg.tfvals[peaktf]) + " Hz", xy=(0,0.9), xycoords='axes fraction', fontsize=14)
        ax6.set_xticks(dg.orivals)
        ax7.set_xlim(-10,325)
        ax6.set_xlabel("Direction (d)", fontsize=20)
        ax6.set_ylabel("Mean DF/F (%)", fontsize=20)
        ax6.yaxis.set_major_locator(MaxNLocator(6))
        
        ax7.errorbar(range(5), dg.response[peakori,1:,nc,0], yerr=dg.response[peakori,1:,nc,1],  fmt='bo-', lw=2)
        ax7.fill_between(range(5), np.repeat(dg.response[0,0,nc,0]+dg.response[0,0,nc,1], 5), np.repeat(dg.response[0,0,nc,0]-dg.response[0,0,nc,1], 5), color='gray', alpha=0.5)
        ax7.axhline(y=dg.response[0,0,nc,0], ls='--', color='k', lw=2)
        ax7.annotate(str(dg.orivals[peakori]) + " Deg", xy=(0,0.9), xycoords='axes fraction', fontsize=14)
        ax7.set_xlim(-0.2, 4.2)
        ax7.set_xticks(range(5))
        ax7.set_xticklabels(dg.tfvals[1:])
        ax7.set_xlabel("Temporal frequency (Hz)", fontsize=20)
        
        subset = dg.sweep_response[(dg.stim_table.orientation==dg.orivals[peakori])&(dg.stim_table.temporal_frequency==dg.tfvals[peaktf])]
        xtime = np.arange(-1*dg.interlength/dg.acquisition_rate, (dg.sweeplength+dg.interlength)/dg.acquisition_rate, 1/dg.acquisition_rate)
        while len(xtime)>len(subset[str(nc)].mean()):
            xtime = np.delete(xtime, -1)
        for index, row in subset.iterrows():
            ax8.plot(xtime, subset[str(nc)][index], lw=2)   
        ax8.set_xlim(-1,3)
        ax8.annotate(str(dg.orivals[peakori])+" Deg / "+str(dg.tfvals[peaktf])+" Hz", xy=(0,0.9), xycoords='axes fraction', fontsize=14)
        ax8.set_xlabel("Time (s)", fontsize=20)
        ax8.set_ylabel("DF/F (%)", fontsize=20)
        ax8.axvspan(0, dg.sweeplength/dg.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
        ax8.yaxis.set_major_locator(MaxNLocator(6))
        ax8.set_title("Trial responses to prefered ori/tf", fontsize=20)   
        
        im = ax9.imshow(dg.response[:,1:,nc,0], cmap='gray', interpolation='none')
        ax9.set_ylabel("Direction (d)", fontsize=20)
        ax9.set_yticks(range(8))
        ax9.set_yticklabels(dg.orivals)
        ax9.set_xlabel("Temporal frequency (Hz)", fontsize=20)
        ax9.set_xticks(range(5))
        ax9.set_xticklabels(dg.tfvals[1:])        
        cbar = plt.colorbar(im, ax=ax9)
        cbar.ax.set_ylabel('DF/F (%)', fontsize=8)
        for t in cbar.ax.get_yticklabels():
            t.set_fontsize(8)
        
        xtime = np.arange(0, nm1.sweeplength/nm1.acquisition_rate, 1/nm1.acquisition_rate)
        while len(xtime)>len(nm1.sweep_response[str(nc)].mean()):
            xtime = np.delete(xtime, -1)        
        for index, row in nm1.sweep_response.iterrows():
            ax10.plot(xtime, nm1.sweep_response[str(nc)][index], lw=2)
        ax10.set_xlabel("Time (s)", fontsize=20)
        ax10.set_ylabel("DF/F", fontsize=20)
        ax10.set_title("Natural Movie 1", fontsize=20, color='red')
        
        temp = np.empty((len(nm1.stim_table), nm1.sweeplength))
        for i in range(len(nm1.stim_table)):
            temp[i,:] = nm1.sweep_response[str(nc)].iloc[i]
        ax11.imshow(temp, cmap='gray', interpolation='none', aspect=40)
        ax11.set_ylabel("Trials", fontsize=20)
        ax11.set_xticks([])
        
        xtime = np.arange(0, nm3.sweeplength/nm3.acquisition_rate, 1/nm3.acquisition_rate)
        while len(xtime)>len(nm3.sweep_response[str(nc)].mean()):
            xtime = np.delete(xtime, -1)        
        for index, row in nm3.sweep_response.iterrows():
            ax12.plot(xtime, nm3.sweep_response[str(nc)][index], lw=2)
        ax12.set_xlabel("Time (s)", fontsize=20)
        ax12.set_ylabel("DF/F", fontsize=20)
        ax12.set_title("Natural Movie Long", fontsize=20, color='blue')
        
        temp = np.empty((len(nm3.stim_table), nm3.sweeplength))
        for i in range(len(nm3.stim_table)):
            temp[i,:] = nm3.sweep_response[str(nc)].iloc[i]
        ax13.imshow(temp, cmap='gray', interpolation='none', aspect=100)
        ax13.set_ylabel("Trials", fontsize=20)
        ax13.set_xticks([])
        
        plt.tick_params(labelsize=16)    
        plt.tight_layout()
        filename = 'Cell_'+str(nc+1)+'_3SA.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()      

def _plot_3sc(lsn, nm1, nm2, save_dir):
    logging.info("Plotting for all cells")
    for nc in range(lsn.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
                         
        plt.figure(nc, figsize=(20,20)) 
        ax1 = plt.subplot2grid((6,6), (0,0), colspan=4) #full trace
        ax2 = plt.subplot2grid((6,6), (0,4))            #histogram of F
        ax11 = plt.subplot2grid((6,6), (1,0), colspan=4)
        ax12 = plt.subplot2grid((6,6), (1,4))
        ax13 = plt.subplot2grid((6,6), (1,5), sharex=ax12, sharey=ax12)
        ax3 = plt.subplot2grid((6,6), (2,0), colspan=3)  #movie 1
        ax4 = plt.subplot2grid((6,6), (3,0), colspan=3)
        ax5 = plt.subplot2grid((6,6), (2,3), colspan=3) #movie 2
        ax6 = plt.subplot2grid((6,6), (3,3), colspan=3)
        ax7 = plt.subplot2grid((6,6), (4,0), colspan=3) #receptive fields
        ax8 = plt.subplot2grid((6,6), (4,3), colspan=3)
        ax9 = plt.subplot2grid((6,6), (5,0), colspan=3)
        ax10 = plt.subplot2grid((6,6), (5,3), colspan=3)
        
        xtime = np.arange(0,np.size(lsn.celltraces,1),1.)
        xtime /= lsn.acquisition_rate
        dif = np.ediff1d(lsn.stim_table.start.values, to_begin=8000, to_end=8000)
        test = np.argwhere(dif>5000)
        ax1.plot(xtime, lsn.celltraces[nc,:])        
        for i in range(len(test)-1): 
            ax1.axvspan(xmin=(lsn.stim_table.start.iloc[test[i]].values/lsn.acquisition_rate), xmax=(lsn.stim_table.end.iloc[test[i+1]-1].values/lsn.acquisition_rate), color='gray', alpha=0.3)
        ax1.axvspan(nm1.stim_table.start.min()/nm1.acquisition_rate, nm1.stim_table.end.max()/nm1.acquisition_rate, ymin=0, ymax=1, color='red', alpha=0.3)
        ax1.axvspan(nm2.stim_table.start.min()/nm2.acquisition_rate, nm2.stim_table.end.max()/nm2.acquisition_rate, ymin=0, ymax=1, color='green', alpha=0.3)
        ax1.set_xlabel("Time (s)", fontsize=20)
        ax1.set_ylabel("Fluorescence", fontsize=20)
        
        ax2.hist(lsn.celltraces[nc,:], bins=70)
        ax2.set_yscale('log')
        ax2.set_xlabel("Fluorescence", fontsize=20)
        ax2.set_ylabel("Count", fontsize=20)
        
        xtime = np.arange(0, np.size(lsn.dxcm), 1.)
        xtime /= lsn.acquisition_rate
        ax11.plot(xtime, lsn.dxcm, color='k')
        ax11.set_xlabel("Time (s)", fontsize=20)
        ax11.set_xlabel("Speed (cm/s)", fontsize=20)
        
        smax = nm1.binned_cells_sp[nc,np.argmax(nm1.binned_cells_sp[nc,:,0]),0] + nm1.binned_cells_sp[nc,np.argmax(nm1.binned_cells_sp[nc,:,0]),1]
        vmax = nm1.binned_cells_vis[nc,np.argmax(nm1.binned_cells_vis[nc,:,0]),0] + nm1.binned_cells_vis[nc,np.argmax(nm1.binned_cells_vis[nc,:,0]),1]
        rmax = np.where(smax>vmax, smax, vmax)
        smin = nm1.binned_cells_sp[nc,np.argmin(nm1.binned_cells_sp[nc,:,0]),0] - nm1.binned_cells_sp[nc,np.argmin(nm1.binned_cells_sp[nc,:,0]),1]
        vmin = nm1.binned_cells_vis[nc,np.argmin(nm1.binned_cells_vis[nc,:,0]),0] - nm1.binned_cells_vis[nc,np.argmin(nm1.binned_cells_vis[nc,:,0]),1]
        rmin = np.where(smin<vmin, smin, vmin)        
        
        ax12.errorbar(nm1.binned_dx_sp[:,0], nm1.binned_cells_sp[nc,:,0], yerr=nm1.binned_cells_sp[nc,:,1], fmt='.', color='k')
        ax12.set_ylim(rmin, rmax)        
        ax12.set_xlabel("Speed (cm/s)", fontsize=20)
        ax12.set_ylabel("DF/F", fontsize=20)
        ax12.set_title("Spontaneous", fontsize=20)
        
        ax13.errorbar(nm1.binned_dx_vis[:,0], nm1.binned_cells_vis[nc,:,0], yerr=nm1.binned_cells_vis[nc,:,1], fmt='.')
        ax13.set_ylim(rmin, rmax)        
        ax13.set_xlabel("Speed (cm/s)", fontsize=20)
        ax13.set_ylabel("DF/F", fontsize=20)
        ax13.set_title("Visual Stimuli", fontsize=20)
        
        xtime = np.arange(0, nm1.sweeplength/nm1.acquisition_rate, 1/nm1.acquisition_rate)
        for index, row in nm1.sweep_response.iterrows():
            ax3.plot(xtime, nm1.sweep_response[str(nc)][index], lw=2)
        ax3.set_xlabel("Time (s)", fontsize=20)
        ax3.set_ylabel("DF/F", fontsize=20)
        ax3.set_title("Natural Movie 1", fontsize=20, color='red')
        
        temp = np.empty((len(nm1.stim_table), nm1.sweeplength))
        for i in range(len(nm1.stim_table)):
            temp[i,:] = nm1.sweep_response[str(nc)].iloc[i]
        ax4.imshow(temp, cmap='gray', interpolation='none', aspect=40)
        ax4.set_ylabel("Trials", fontsize=20)
        ax4.set_xticks([])
        
        xtime = np.arange(0, nm2.sweeplength/nm2.acquisition_rate, 1/nm2.acquisition_rate)
        for index, row in nm2.sweep_response.iterrows():
            ax5.plot(xtime, nm2.sweep_response[str(nc)][index], lw=2)
        ax5.set_xlabel("Time (s)", fontsize=20)
        ax5.set_ylabel("DF/F", fontsize=20)
        ax5.set_title("Natural Movie 2", fontsize=20, color='green')
        
        temp = np.empty((len(nm2.stim_table), nm2.sweeplength))
        for i in range(len(nm2.stim_table)):
            temp[i,:] = nm2.sweep_response[str(nc)].iloc[i]
        ax6.imshow(temp, cmap='gray', interpolation='none', aspect=40)
        ax6.set_ylabel("Trials", fontsize=20)
        ax6.set_xticks([])
        
        vMax = np.where(np.amax(lsn.receptive_field[:,:,nc,0])>np.amax(lsn.receptive_field[:,:,nc,1]), np.amax(lsn.receptive_field[:,:,nc,0]), np.amax(lsn.receptive_field[:,:,nc,1]))
        vMin = np.where(np.amin(lsn.receptive_field[:,:,nc,0])<np.amin(lsn.receptive_field[:,:,nc,1]), np.amin(lsn.receptive_field[:,:,nc,0]), np.amin(lsn.receptive_field[:,:,nc,1]))

        imon = ax7.imshow(lsn.receptive_field[:,:,nc,0], cmap='gray', interpolation='None', vmin=vMin, vmax=vMax)
        ax7.set_title("ON", fontsize=20)
        ax7.set_xticks([])
        ax7.set_yticks([])
        cbar = plt.colorbar(imon, ax=ax7, fraction=0.046, pad=0.04)
        cbar.ax.set_ylabel('DF/F (%)', fontsize=10)
        for t in cbar.ax.get_yticklabels():
            t.set_fontsize(8)
        
        imoff = ax8.imshow(lsn.receptive_field[:,:,nc,1], cmap='gray', interpolation='None', vmin=vMin, vmax=vMax)
        ax8.set_title("OFF", fontsize=20)
        ax8.set_xticks([])
        ax8.set_yticks([])
        cbar = plt.colorbar(imoff, ax=ax8, fraction=0.046, pad=0.04)
        cbar.ax.set_ylabel('DF/F (%)', fontsize=10)
        for t in cbar.ax.get_yticklabels():
            t.set_fontsize(8)
        
        zon = (lsn.receptive_field[:,:,nc,0]-np.mean(lsn.receptive_field[:,:,nc,0]))/np.std(lsn.receptive_field[:,:,nc,0])
        zon = np.where(abs(zon)>2, zon, 0)
        Vmax_on = np.where(abs(np.amax(zon))>abs(np.amin(zon)), np.amax(zon), -1*np.amin(zon))
        zoff = (lsn.receptive_field[:,:,nc,1]-np.mean(lsn.receptive_field[:,:,nc,1]))/np.std(lsn.receptive_field[:,:,nc,1])
        zoff = np.where(abs(zoff)>2, zoff, 0)
        Vmax_off = np.where(abs(np.amax(zoff))>abs(np.amin(zoff)), np.amax(zoff), -1*np.amin(zoff))
        Vmax = np.where(Vmax_on>Vmax_off, Vmax_on, Vmax_off)
        imzon = ax9.imshow(zon, cmap='RdBu_r', interpolation='none', vmin=-1*Vmax, vmax=Vmax)
        ax9.set_title("On Z-score", fontsize=20)
        ax9.set_xticks([])
        ax9.set_yticks([])
        cbar = plt.colorbar(imzon, ax=ax9, fraction=0.046, pad=0.04)
        
        zoff = (lsn.receptive_field[:,:,nc,1]-np.mean(lsn.receptive_field[:,:,nc,1]))/np.std(lsn.receptive_field[:,:,nc,1])
        zoff = np.where(abs(zoff)>2, zoff, 0)
        imzoff = ax10.imshow(zoff, cmap='RdBu', interpolation='none', vmin=-1*Vmax, vmax=Vmax)
        ax10.set_title("Off Z-score", fontsize=20)
        ax10.set_xticks([])
        ax10.set_yticks([])
        cbar = plt.colorbar(imzoff, ax=ax10, fraction=0.046, pad=0.04)
        
        plt.tick_params(labelsize=16)    
        plt.tight_layout()
        filename = 'Cell_'+str(nc+1)+'_3SC.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()

def _plot_3sb(sg, nm1, ns, save_dir):
    logging.info("Plotting for all cells")
    for nc in range(sg.numbercells):
        if np.mod(nc,20)==0:
            logging.info("Cell #%s", str(nc))
        plt.figure(nc, figsize=(20,24)) 
        ax1 = plt.subplot2grid((6,6), (0,0), colspan=4) #full trace
        ax2 = plt.subplot2grid((6,6), (0,4))            #histogram of F
        ax14 = plt.subplot2grid((6,6), (1,4))           #speed tuning
        ax16 = plt.subplot2grid((6,6), (1,5), sharex=ax14, sharey=ax14)
        ax15 = plt.subplot2grid((6,6), (1,0), colspan=4)
        ax3 = plt.subplot2grid((6,6),(2,0), colspan=2)  #Ori tuning
        ax4 = plt.subplot2grid((6,6),(2,2), colspan=2)  #sf tuning
        ax13 = plt.subplot2grid((6,6),(2,4), colspan=2)  #response at peak
        ax5 = plt.subplot2grid((6,6), (3,0))
        ax6 = plt.subplot2grid((6,6), (3,1))
        ax7 = plt.subplot2grid((6,6), (3,2))
        ax8 = plt.subplot2grid((6,6), (3,3))
        ax9 = plt.subplot2grid((6,6),(4,0), colspan=3)  #movie
        ax10 = plt.subplot2grid((6,6),(5,0), colspan=3)
        ax11 = plt.subplot2grid((6,6),(4,3), colspan=2) #natural scenes
        ax12 = plt.subplot2grid((6,6),(5,3), colspan=2)
        
        
        xtime = np.arange(0,np.size(sg.celltraces,1),1.)
        xtime /= sg.acquisition_rate
        dif = np.ediff1d(sg.stim_table.start.values, to_begin=8000, to_end=8000)
        test = np.argwhere(dif>5000)
        ax1.plot(xtime, sg.celltraces[nc,:])        
        for i in range(len(test)-1): 
            ax1.axvspan(xmin=(sg.stim_table.start.iloc[test[i]].values/sg.acquisition_rate), xmax=(sg.stim_table.end.iloc[test[i+1]-1].values/sg.acquisition_rate), color='gray', alpha=0.3)
        ax1.axvspan(nm1.stim_table.start.min()/nm1.acquisition_rate, nm1.stim_table.end.max()/nm1.acquisition_rate, ymin=0, ymax=1, color='red', alpha=0.3)
        dif = np.ediff1d(ns.stim_table.start.values, to_begin=8000, to_end=8000)
        test = np.argwhere(dif>5000)
        for i in range(len(test)-1): 
            ax1.axvspan(xmin=(ns.stim_table.start.iloc[test[i]].values/ns.acquisition_rate), xmax=(ns.stim_table.end.iloc[test[i+1]-1].values/ns.acquisition_rate), color='blue', alpha=0.3)
        ax1.set_xlabel("Time (s)", fontsize=20)
        ax1.set_ylabel("Fluorescence", fontsize=20)
        
        ax2.hist(sg.celltraces[nc,:], bins=70)
        ax2.set_yscale('log')
        ax2.set_xlabel("Fluorescence", fontsize=20)
        ax2.set_ylabel("Count", fontsize=20)
        
        xtime = np.arange(0, np.size(sg.dxcm), 1.)
        xtime /= sg.acquisition_rate
        ax15.plot(xtime, sg.dxcm, color='k')
        ax15.set_xlabel("Time (s)", fontsize=20)
        ax15.set_xlabel("Speed (cm/s)", fontsize=20)
        
        peakori = sg.peak.ori_sg[nc]
        peaksf = sg.peak.sf_sg[nc]
        ax3.errorbar(sg.orivals, sg.response[:,peaksf,0,nc,0], yerr=sg.response[:,peaksf,0,nc,1], color='blue', fmt='o-', lw=2)
        ax3.errorbar(sg.orivals, sg.response[:,peaksf,1,nc,0], yerr=sg.response[:,peaksf,1,nc,1], color='cornflowerblue', fmt='o-', lw=2)
        ax3.errorbar(sg.orivals, sg.response[:,peaksf,2,nc,0], yerr=sg.response[:,peaksf,2,nc,1], color='steelblue', fmt='o-', lw=2)
        ax3.errorbar(sg.orivals, sg.response[:,peaksf,3,nc,0], yerr=sg.response[:,peaksf,3,nc,1], color='lightskyblue', fmt='o-', lw=2)
        ax3.fill_between(sg.orivals, np.repeat(sg.response[0,0,0,nc,0]+sg.response[0,0,0,nc,1], sg.number_ori), np.repeat(sg.response[0,0,0,nc,0]-sg.response[0,0,0,nc,1], sg.number_ori), color='gray', alpha=0.5)
        ax3.axhline(y=sg.response[0,0,0,nc,0], ls='--', color='k', lw=2)
        ax3.set_xlim(-10, 160)
        ax3.set_xticks(sg.orivals)
        ax3.set_xlabel("Orientation (d)", fontsize=20)
        ax3.set_ylabel("DF/F (%)", fontsize=20)
        
        ax4.errorbar(range(5), sg.response[peakori,1:,0,nc,0], yerr=sg.response[peakori,1:,0,nc,1], color='blue', fmt='o-', lw=2)
        ax4.errorbar(range(5), sg.response[peakori,1:,1,nc,0], yerr=sg.response[peakori,1:,1,nc,1], color='cornflowerblue', fmt='o-', lw=2)
        ax4.errorbar(range(5), sg.response[peakori,1:,2,nc,0], yerr=sg.response[peakori,1:,2,nc,1], color='steelblue', fmt='o-', lw=2)
        ax4.errorbar(range(5), sg.response[peakori,1:,3,nc,0], yerr=sg.response[peakori,1:,3,nc,1], color='lightskyblue', fmt='o-', lw=2)
        ax4.fill_between(range(5), np.repeat(sg.response[0,0,0,nc,0]+sg.response[0,0,0,nc,1], 5), np.repeat(sg.response[0,0,0,nc,0]-sg.response[0,0,0,nc,1], 5), color='gray', alpha=0.5)
        ax4.axhline(y=sg.response[0,0,0,nc,0], ls='--', color='k', lw=2)
        ax4.set_xlim(-0.2, 4.2)
        ax4.set_xticks(range(5))
        ax4.set_xticklabels(sg.sfvals[1:])
        ax4.set_xlabel("Spatial frequency (cpd)", fontsize=20)
        
        xtime = np.arange(-1*sg.interlength/sg.acquisition_rate, (sg.sweeplength+sg.interlength)/sg.acquisition_rate, 1/sg.acquisition_rate)
        peakori = sg.peak.ori_sg[nc]
        peaksf = sg.peak.sf_sg[nc]
        peakphase = sg.peak.phase_sg[nc]
        subset = sg.sweep_response[(sg.stim_table.orientation==sg.orivals[peakori])&(sg.stim_table.spatial_frequency==sg.sfvals[peaksf])&(sg.stim_table.phase==sg.phasevals[peakphase])]
        subset_p =  subset[str(nc)].mean() + (subset[str(nc)].std()/np.sqrt(len(subset[str(nc)])))
        subset_n =  subset[str(nc)].mean() - (subset[str(nc)].std()/np.sqrt(len(subset[str(nc)])))
        try:
            ax13.fill_between(xtime, subset_p, subset_n, color='b', alpha=0.5)
        except:
            xtime = xtime[:-1]
            ax13.fill_between(xtime, subset_p, subset_n, color='b', alpha=0.5)
        blank = sg.sweep_response[(sg.stim_table.orientation==0)&(sg.stim_table.spatial_frequency==0)&(sg.stim_table.phase==0)]
        blank_p = blank[str(nc)].mean() + (blank[str(nc)].std()/np.sqrt(len(blank[str(nc)])))
        blank_n = blank[str(nc)].mean() - (blank[str(nc)].std()/np.sqrt(len(blank[str(nc)])))        
        ax13.fill_between(xtime, blank_p, blank_n, color='gray', alpha=0.5)        
        ax13.plot(xtime, subset[str(nc)].mean(), color='b', lw=2)        
        ax13.plot(xtime, blank[str(nc)].mean(), color='k', lw=2)
        ax13.axvspan(0, sg.sweeplength/sg.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)        
        ax13.yaxis.set_major_locator(MaxNLocator(4)) 
        ax13.set_xlabel("Time (s)", fontsize=20)
        ax13.set_ylabel("DF/F (%)", fontsize=20)
        
        Vmax = sg.response[:,1:,:,nc,0].max()
        ax5.imshow(sg.response[:,1:,0,nc,0], cmap='gray', interpolation='none', vmin=0, vmax=Vmax)
        ax5.set_ylabel("Orientation (d)", fontsize=20)
        ax5.set_yticks(range(6))
        ax5.set_yticklabels(sg.orivals)
        ax5.set_xlabel("Spatial frequency (cpd)", fontsize=20)
        ax5.set_xticks(range(5))
        ax5.set_xticklabels(sg.sfvals[1:])
        ax5.set_title("Phase 0.0", color='blue', fontsize=20)
        
        ax6.imshow(sg.response[:,1:,1,nc,0], cmap='gray', interpolation='none', vmin=0, vmax=Vmax)
        ax6.set_xlabel("Spatial frequency (cpd)", fontsize=20)
        ax6.set_xticks(range(5))
        ax6.set_xticklabels(sg.sfvals[1:])
        ax6.set_yticks(range(6))
        ax6.set_yticklabels(sg.orivals)
        ax6.set_title("Phase 0.25", color='cornflowerblue', fontsize=20)
        
        ax7.imshow(sg.response[:,1:,2,nc,0], cmap='gray', interpolation='none', vmin=0, vmax=Vmax)
        ax7.set_xlabel("Spatial frequency (cpd)", fontsize=20)
        ax7.set_xticks(range(5))
        ax7.set_xticklabels(sg.sfvals[1:])
        ax7.set_yticks(range(6))
        ax7.set_yticklabels(sg.orivals)
        ax7.set_title("Phase 0.5", color='steelblue', fontsize=20)
        
        ax8.imshow(sg.response[:,1:,3,nc,0], cmap='gray', interpolation='none', vmin=0, vmax=Vmax)
        ax8.set_xlabel("Spatial frequency (cpd)", fontsize=20)
        ax8.set_xticks(range(5))
        ax8.set_xticklabels(sg.sfvals[1:])
        ax8.set_yticks(range(6))
        ax8.set_yticklabels(sg.orivals)
        ax8.set_title("Phase 0.75", color='lightskyblue', fontsize=20)
        
        xtime = np.arange(0, nm1.sweeplength/nm1.acquisition_rate, 1/nm1.acquisition_rate)
        while len(xtime)>nm1.sweeplength:
            xtime = np.delete(xtime, -1)            
        for index, row in nm1.sweep_response.iterrows():
            ax9.plot(xtime, nm1.sweep_response[str(nc)][index], lw=2)
        ax9.set_xlabel("Time (s)", fontsize=20)
        ax9.set_ylabel("DF/F", fontsize=20)
        ax9.set_title("Natural Movie 1", fontsize=20, color='red')
        
        temp = np.empty((len(nm1.stim_table), nm1.sweeplength))
        for i in range(len(nm1.stim_table)):
            temp[i,:] = nm1.sweep_response[str(nc)].iloc[i]
        ax10.imshow(temp, cmap='gray', interpolation='none', aspect=40)
        ax10.set_ylabel("Trials", fontsize=20)
        ax10.set_xticks([])
        
        temp = np.copy(ns.response[1:,nc,:2])
        scene_response = pd.DataFrame(temp, columns=('response','error'))
        scene_response = scene_response.sort(columns='response', ascending=False)
        ax11.errorbar(range(ns.number_scenes-1), scene_response.response, yerr=scene_response.error, fmt='o', color='k')
        ax11.fill_between(range(ns.number_scenes-1), np.repeat(ns.response[0,nc,0]+ns.response[0,nc,1], ns.number_scenes-1), np.repeat(ns.response[0,nc,0]-ns.response[0,nc,1], ns.number_scenes-1), color='gray', alpha=0.3)
        ax11.axhline(y=ns.response[0,nc,0], ls='--', lw=2, color='k')
        ax11.set_xlim(-2,120)
        ax11.set_title("Natural Scenes", fontsize=20, color='blue')
        ax11.set_xlabel("Scene", fontsize=20)
        ax11.set_ylabel("DF/F (%)", fontsize=20)
        
        xtime = np.arange(-1*ns.interlength/ns.acquisition_rate, (ns.sweeplength+ns.interlength)/ns.acquisition_rate, 1/ns.acquisition_rate)
        nsp = np.argmax(ns.response[1:,nc,0])
        subset_response = ns.sweep_response[ns.stim_table.frame==nsp]               
        subset_response_p = subset_response[str(nc)].mean() + (subset_response[str(nc)][:].std()/np.sqrt(len(subset_response[str(nc)])))
        subset_response_n = subset_response[str(nc)].mean() - (subset_response[str(nc)][:].std()/np.sqrt(len(subset_response[str(nc)])))
        try:        
            ax12.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
        except:
            xtime = xtime[:-1]
            ax12.fill_between(xtime, subset_response_p, subset_response_n, color='b', alpha=0.5)
        blank = ns.sweep_response[ns.stim_table.frame==-1]
        blank_p = blank[str(nc)].mean() + (blank[str(nc)].std()/np.sqrt(len(blank[str(nc)])))
        blank_n = blank[str(nc)].mean() - (blank[str(nc)].std()/np.sqrt(len(blank[str(nc)])))        
        ax12.fill_between(xtime, blank_p, blank_n, color='gray', alpha=0.5)          
        ax12.plot(xtime, subset_response[str(nc)].mean(), color='b', lw=2)
        ax12.plot(xtime, blank[str(nc)].mean(), color='k', lw=2)                 
        ax12.axvspan(0, ns.sweeplength/ns.acquisition_rate ,ymin=0, ymax=1, facecolor='gray', alpha=0.3)
        ax12.yaxis.set_major_locator(MaxNLocator(4)) 
        ax12.set_xlabel("Time (s)", fontsize=20)
        ax12.set_ylabel("DF/F (%)", fontsize=20)
        
        
        plt.tick_params(labelsize=16)    
        plt.tight_layout()
        filename = 'Cell_'+str(nc+1)+'_3SB.png'
        fullfilename = os.path.join(save_dir, filename) 
        plt.savefig(fullfilename)   
        plt.close()


def plot_running_a(dg, nm1, nm3, save_dir):
    logging.info("Plotting running data summary")
    nc = -1
    plt.figure(1, figsize=(10,8))
    ax1 = plt.subplot2grid((4,4),(0,0), colspan=3)
    ax2 = plt.subplot2grid((4,4),(0,3))
    ax3 = plt.subplot2grid((4,4),(1,0))
    ax4 = plt.subplot2grid((4,4),(1,1))
    ax5 = plt.subplot2grid((4,4),(1,2))
    ax6 = plt.subplot2grid((4,4), (2,0), colspan=2)
    ax7 = plt.subplot2grid((4,4), (3,0), colspan=2)
    ax8 = plt.subplot2grid((4,4), (2,2), colspan=2)
    ax9 = plt.subplot2grid((4,4), (3,2), colspan=2)
    
    xtime = np.arange(0, np.size(dg.dxcm), 1.)
    xtime /= dg.acquisition_rate
    dif = np.ediff1d(dg.stim_table.start.values, to_begin=8000, to_end=8000)
    test = np.argwhere(dif>5000)
    ax1.plot(xtime, dg.dxcm, color='k')        
    for i in range(len(test)-1): 
        ax1.axvspan(xmin=(dg.stim_table.start.iloc[test[i]].values/dg.acquisition_rate), xmax=(dg.stim_table.end.iloc[test[i+1]-1].values/dg.acquisition_rate), color='gray', alpha=0.3)
    ax1.axvspan(xmin=nm1.stim_table.start.min()/nm1.acquisition_rate, xmax=((nm1.stim_table.start.max()+nm1.sweeplength)/nm1.acquisition_rate), color='red', alpha=0.3)
    dif = np.ediff1d(nm3.stim_table.start.values, to_begin=8000, to_end=8000)
    test = np.argwhere(dif>5000)
    for i in range(len(test)-1): 
        ax1.axvspan(xmin=(nm3.stim_table.start.iloc[test[i]].values/nm3.acquisition_rate), xmax=((nm3.stim_table.end.iloc[test[i+1]-1].values+nm3.sweeplength)/nm3.acquisition_rate), color='blue', alpha=0.3)
    ax1.set_xlabel("Time (s)", fontsize=20)
    ax1.set_ylabel("Speed (cm/s)", fontsize=20)
    
    dx = dg.dxcm[np.logical_not(np.isnan(dg.dxcm))]
    ax2.hist(dx, bins=80, range=(-20,100), color='gray')
    ax2.set_xlabel("Speed (cm/s)", fontsize=20)
    
    run_peak = np.where(dg.response[:,1:,nc,0] == np.nanmax(dg.response[:,1:,nc,0]))
    peakori = run_peak[0][0]
    peaktf = run_peak[1][0]+1
    
    ax3.errorbar(dg.orivals, dg.response[:,peaktf,nc,0], yerr=dg.response[:,peaktf,nc,1], fmt='b.-', lw=2)
    ax3.fill_between(dg.orivals, np.repeat(dg.response[0,0,nc,0]+dg.response[0,0,nc,1], dg.number_ori), np.repeat(dg.response[0,0,nc,0]-dg.response[0,0,nc,1], dg.number_ori), color='gray', alpha=0.5)
    ax3.axhline(y=dg.response[0,0,nc,0], ls='--', color='k')
    ax3.annotate(str(dg.tfvals[peaktf]) + " Hz", xy=(0,0.9), xycoords='axes fraction', fontsize=14)
    ax3.set_xtick = (dg.orivals)
    ax3.set_xlabel("Direction (deg)", fontsize=20)
    ax3.set_ylabel("Speed (cm/s)", fontsize=20)
    ax3.yaxis.set_major_locator(MaxNLocator(6))
    
    ax4.errorbar(dg.tfvals[1:], dg.response[peakori,1:,nc,0], yerr=dg.response[peakori,1:,nc,1], fmt='b.-', lw=2)
    ax4.fill_between(dg.tfvals[1:], np.repeat(dg.response[0,0,nc,0]+dg.response[0,0,nc,1], dg.number_tf-1), np.repeat(dg.response[0,0,nc,0]-dg.response[0,0,nc,1], dg.number_tf-1), color='gray', alpha=0.5)
    ax4.axhline(y=dg.response[0,0,nc,0], ls='--', color='k')  
    ax4.annotate(str(dg.orivals[peakori]) + " Deg", xy=(0,0.9), xycoords='axes fraction', fontsize=14)
    ax4.set_xticks = (dg.tfvals[1:])
    ax4.set_xlabel("Temporal frequency (Hz)", fontsize=20)
    ax4.yaxis.set_major_locator(MaxNLocator(6))
    
    im = ax5.imshow(dg.response[:,1:,nc,0], cmap='gray', interpolation='none')
    ax5.set_ylabel("Direction", fontsize=16)
    ax5.set_xlabel("TF", fontsize=16)
    ax5.set_yticks(range(dg.number_ori))
    ax5.set_yticklabels(list(dg.orivals.astype(int).astype(str)))
    ax5.set_xticks(range(dg.number_tf-1))
    ax5.set_xticklabels(list(dg.tfvals[1:].astype(int).astype(str)))
    cbar = plt.colorbar(im, ax=ax5)
    cbar.ax.set_ylabel('Speed (cm/s)', fontsize=8)
    for t in cbar.ax.get_yticklabels():
        t.set_fontsize(8)
    
    xtime = np.arange(0, nm1.sweeplength/nm1.acquisition_rate, 1/nm1.acquisition_rate)
    while len(xtime)>len(nm1.sweep_response['dx'].mean()):
        xtime = np.delete(xtime, -1)        
    for index, row in nm1.sweep_response.iterrows():
        ax6.plot(xtime, nm1.sweep_response['dx'][index], lw=2)
    ax6.set_xlabel("Time (s)", fontsize=20)
    ax6.set_ylabel("DF/F", fontsize=20)
    ax6.set_title("Natural Movie 1", fontsize=20, color='red')
    
    temp = np.empty((len(nm1.stim_table), nm1.sweeplength))
    for i in range(len(nm1.stim_table)):
        temp[i,:] = nm1.sweep_response['dx'].iloc[i][:,0]
    ax7.imshow(temp, cmap='gray', interpolation='none', aspect=40)
    ax7.set_ylabel("Trials", fontsize=20)
    ax7.set_xticks([])
    
    xtime = np.arange(0, nm3.sweeplength/nm3.acquisition_rate, 1/nm3.acquisition_rate)
    while len(xtime)>len(nm3.sweep_response['dx'].mean()):
        xtime = np.delete(xtime, -1)        
    for index, row in nm3.sweep_response.iterrows():
        ax8.plot(xtime, nm3.sweep_response['dx'][index], lw=2)
    ax8.set_xlabel("Time (s)", fontsize=20)
    ax8.set_ylabel("DF/F", fontsize=20)
    ax8.set_title("Natural Movie Long", fontsize=20, color='blue')
    
    temp = np.empty((len(nm3.stim_table), nm3.sweeplength))
    for i in range(len(nm3.stim_table)):
        temp[i,:] = nm3.sweep_response['dx'].iloc[i][:,0]
    ax9.imshow(temp, cmap='gray', interpolation='none', aspect=100)
    ax9.set_ylabel("Trials", fontsize=20)
    ax9.set_xticks([])

    plt.tick_params(labelsize=16)
    plt.tight_layout()
    plt.suptitle("Running Summary", fontsize=20)
    plt.subplots_adjust(top=0.9)
    filename = 'Running Summary.png'
    fullfilename = os.path.join(save_dir, filename) 
    plt.savefig(fullfilename)   
    plt.close()              



# Copyright 2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.


import scipy.stats as st
import numpy as np
import pandas as pd
from allensdk.brain_observatory.stimulus_analysis import StimulusAnalysis
import logging

class NaturalScenes(StimulusAnalysis):
    """ Perform tuning analysis specific to natural scenes stimulus. 

    Parameters
    ----------
    data_set: BrainObservatoryNwbDataSet object
    """

    _log = logging.getLogger('allensdk.brain_observatory.natural_scenes')    
    
    def __init__(self, data_set, **kwargs):
        super(NaturalScenes, self).__init__(data_set, **kwargs)
        self.stim_table = self.data_set.get_stimulus_table('natural_scenes')
        self.number_scenes = len(np.unique(self.stim_table.frame))
        self.sweeplength = self.stim_table.end.iloc[1] - self.stim_table.start.iloc[1]
        self.interlength = 4 * self.sweeplength
        self.extralength = self.sweeplength
        self.sweep_response, self.mean_sweep_response, self.pval = self.get_sweep_response()
        self.response = self.get_response()
        self.peak = self.get_peak()
        
    def get_response(self):
        ''' Computes the mean response for each cell to each stimulus condition.  Return is 
        a (# scenes, # cells, 3) np.ndarray.  The final dimension
        contains the mean response to the condition (index 0), standard error of the mean of the response
        to the condition (index 1), and p value of the response to that condition (index 3).

        Returns
        -------
        Numpy array storing mean responses.
        '''
        NaturalScenes._log.info("Calculating mean responses")
        
        response = np.empty((self.number_scenes, self.numbercells+1, 3))
        
        def ptest(x):
            return len(np.where(x<(0.05/(self.number_scenes-1)))[0])
            
        for ns in range(self.number_scenes):
            subset_response = self.mean_sweep_response[self.stim_table.frame==(ns-1)]
            subset_pval = self.pval[self.stim_table.frame==(ns-1)]            
            response[ns,:,0] = subset_response.mean(axis=0)
            response[ns,:,1] = subset_response.std(axis=0)/np.sqrt(len(subset_response))
            response[ns,:,2] = subset_pval.apply(ptest, axis=0)
        
        return response
    
    def get_peak(self):    
        ''' Computes metrics about peak response condition for each cell.
        
        Returns
        -------
        Pandas data frame with the following fields ('_ns' suffix is for
        natural scene):
            * scene_ns (scene number)
            * response_reliability_ns
            * peak_dff_ns (peak dF/F)
            * ptest_ns
            * p_run_ns
            * run_modulation_ns
            * time_to_peak_ns
            * duration_ns
        '''
        NaturalScenes._log.info('Calculating peak response properties')
        peak = pd.DataFrame(index=range(self.numbercells), columns=('scene_ns', 'response_reliability_ns','peak_dff_ns', 'ptest_ns', 'p_run_ns', 'run_modulation_ns', 'time_to_peak_ns','duration_ns','cell_specimen_id'))
        cids = self.data_set.get_cell_specimen_ids()

        for nc in range(self.numbercells):
            nsp = np.argmax(self.response[1:,nc,0])
            peak.cell_specimen_id.iloc[nc] = cids[nc]
            peak.scene_ns[nc] = nsp
            peak.response_reliability_ns[nc] = self.response[nsp+1,nc,2]/0.50 #assume 50 trials
            peak.peak_dff_ns[nc] = self.response[nsp+1,nc,0]
            subset = self.mean_sweep_response[self.stim_table.frame==nsp]
            subset_stat = subset[subset.dx<2]
            subset_run = subset[subset.dx>=2]
            if (len(subset_run)>5) & ( len(subset_stat)>5):
                (_, peak.p_run_ns[nc]) = st.ks_2samp(subset_run[str(nc)], subset_stat[str(nc)])
                peak.run_modulation_ns[nc] = subset_run[str(nc)].mean()/subset_stat[str(nc)].mean()
            else:
                peak.p_run_ns[nc] = np.NaN
                peak.run_modulation_ns[nc] = np.NaN
            groups = []
            for im in range(self.number_scenes):
                subset = self.mean_sweep_response[self.stim_table.frame==(im-1)]
                groups.append(subset[str(nc)].values)
            (_,peak.ptest_ns[nc]) = st.f_oneway(*groups)
            test = self.sweep_response[self.stim_table.frame==nsp][str(nc)].mean()
            peak.time_to_peak_ns[nc] = (np.argmax(test) - self.interlength)/self.acquisition_rate
            test2 = np.where(test<(test.max()/2))[0]
            try:          
                peak.duration_ns[nc] = np.ediff1d(test2).max()/self.acquisition_rate
            except:
                pass

        return peak



from configure_model import configure_model
import allensdk.core.json_utilities as ju
import numpy as np
import re, os, logging

MODEL_CONFIG_DIR = "/local1/stash/glif_clean/test/model_config_files/"

MODEL_CONFIG_FILES = [    
        os.path.join(MODEL_CONFIG_DIR, "329552531Rorb-IRES2-CrePOS/329552531Rorb-IRES2-CrePOS_LIF_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "329552531Rorb-IRES2-CrePOS/329552531Rorb-IRES2-CrePOS_LIF_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "369697038Nr5a1-CrePOS/369697038Nr5a1-CrePOS_LIF_R_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "369697038Nr5a1-CrePOS/369697038Nr5a1-CrePOS_LIF_R_ASC_AT_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "369697038Nr5a1-CrePOS/369697038Nr5a1-CrePOS_LIF_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "369697038Nr5a1-CrePOS/369697038Nr5a1-CrePOS_LIF_R_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "369697038Nr5a1-CrePOS/369697038Nr5a1-CrePOS_LIF_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485912047Cux2-CreERT2POS/485912047Cux2-CreERT2POS_LIF_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485912047Cux2-CreERT2POS/485912047Cux2-CreERT2POS_LIF_ASC_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485912047Cux2-CreERT2POS/485912047Cux2-CreERT2POS_LIF_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485912047Cux2-CreERT2POS/485912047Cux2-CreERT2POS_LIF_ASC_mlin_model_config.json"),

        os.path.join(MODEL_CONFIG_DIR, "485455285Sst-IRES-CrePOS/485455285Sst-IRES-CrePOS_LIF_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485455285Sst-IRES-CrePOS/485455285Sst-IRES-CrePOS_LIF_ASC_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485455285Sst-IRES-CrePOS/485455285Sst-IRES-CrePOS_LIF_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "485455285Sst-IRES-CrePOS/485455285Sst-IRES-CrePOS_LIF_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_ASC_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_ASC_AT_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_ASC_AT_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_PWL_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_ASC_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_R_mlin_model_config.json"),
        os.path.join(MODEL_CONFIG_DIR, "476106176Sst-IRES-CrePOS/476106176Sst-IRES-CrePOS_LIF_ASC_PWL_mlin_model_config.json")
]

PREP_DIR = "/local1/stash/allensdk/allensdk/model/glif/test/preprocess/" 
OUT_DIR = "/local1/stash/allensdk/allensdk/model/glif/test/model_config/"
METHOD_CONFIG_DIR = "/local1/stash/allensdk/allensdk/model/glif/method_configurations/"

class ComparisonException( Exception ):
    def __init__(self, message, key):
        super(ComparisonException, self).__init__(message)
        self.key = key
        
def cmpdict(d1, d2):
    k1 = set([ unicode(k) for k in d1.keys() ])
    k2 = set([ unicode(k) for k in d2.keys() ])

    keys = k1 & k2
    disjoint_keys = k1 ^ k2
    
    if len(disjoint_keys) > 0:
        logging.warning("keys NOT in both dicts: %s" % str(disjoint_keys))
        
    for k in keys:
        v1 = d1[k]
        v2 = d2[k]

        try:
            if isinstance(v1, dict):
                cmpdict(v1, v2)
            elif isinstance(v1, list):
                if len(v1) != len(v2):
                    raise ComparisonException("", k)

                for i in range(len(v1)):
                    if v1[i] != v2[i]:
                        raise
            elif d1[k] != d2[k]:
                raise
        except:
            raise ComparisonException("%s: %s vs %s" % (k, str(d1[k]), str(d2[k])), k)
                            

def test_configure_model():
    p = re.compile("(\d+)(.*?)_(.*)_mlin_model_config.json")

    if not os.path.exists(OUT_DIR):
        os.makedirs(OUT_DIR)

    for mcf in MODEL_CONFIG_FILES:
        fname = os.path.basename(mcf)
        m = p.match(fname)
        
        sid, cre, config =  m.groups()

        method_config_file = os.path.join(METHOD_CONFIG_DIR, "%s.json" % config)
        prep_file = os.path.join(PREP_DIR, "%s_preprocessed_dict.json" % sid)
        out_file = os.path.join(OUT_DIR, "%s_%s_model_config.json" % (sid, config))

        if os.path.exists(prep_file):
            print "testing", prep_file
            out_config = configure_model(ju.read(method_config_file), 
                                         ju.read(prep_file))

            ju.write(out_file, out_config)

            test_config = ju.read(mcf)

            cmpdict(test_config['neuron'], out_config['neuron'])
            try:
                cmpdict(test_config['optimizer'], out_config['optimizer'])
            except ComparisonException, e:
                if e.key != "param_fit_names":
                    raise e
                

        else:
            logging.error("preprocessor file %s does not exist" % prep_file)


if __name__ == "__main__": test_configure_model()
    
    



# Copyright 2016 Allen Institute for Brain Science
# This file is part of Allen SDK.
#
# Allen SDK is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# Allen SDK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Allen SDK.  If not, see <http://www.gnu.org/licenses/>.


from allensdk.api.queries.brain_observatory_api \
    import BrainObservatoryApi


import pytest
from mock import patch, MagicMock


@pytest.fixture
def bo_api():
    bo = BrainObservatoryApi('http://testwarehouse:9000')
    bo.json_msg_query = MagicMock(name='json_msg_query')
    
    return bo


@pytest.fixture
def bo_api_save_ophys():
    bo = BrainObservatoryApi('http://testwarehouse:9000')
    bo.json_msg_query = \
        MagicMock(name='json_msg_query',
                  return_value=[ {'download_link': '/url/path/to/file' } ])
    
    return bo


@pytest.fixture
def mock_containers():
    containers = [
        { 'targeted_structure': { 'acronym': 'CBS' },
          'imaging_depth': 100,
          'specimen': {
            'donor': { 'transgenic_lines': [ { 'name': 'Shiny' }] } }
        },
        { 'targeted_structure': { 'acronym': 'NBC' },
          'imaging_depth': 200,
          'specimen': {
            'donor': { 'transgenic_lines': [ { 'name': 'Don' }] } }
        }
    ]
    
    return containers


@pytest.fixture
def mock_ophys_experiments():
    containers = [
        { 'experiment_container_id': 1,
          'targeted_structure': { 'acronym': 'CBS' },
          'imaging_depth': 100,
          'specimen': { 'donor': {
              'transgenic_lines': [ { 'name': 'Shiny' }] } },
          'stimulus_name': 'three_session_B'
        },
        { 'experiment_container_id': 2,
          'targeted_structure': { 'acronym': 'NBC' },
          'imaging_depth': 200,
          'specimen': { 'donor': {
              'transgenic_lines': [ { 'name': 'Don' }] } },
          'stimulus_name': 'three_session_C'
        }
    ]
    
    return containers


@pytest.fixture
def mock_specimens():
    specimens = [ 'stub']
    
    return specimens
    
    
def test_list_isi_experiments(bo_api):
    bo_api.list_isi_experiments()
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::IsiExperiment,rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_isi_experiments(bo_api):      
    isi_experiment_id = 503316697     
    bo_api.get_isi_experiments(isi_experiment_id)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::IsiExperiment,rma::criteria,[id$in503316697],"
        "rma::include,"
        "experiment_container(ophys_experiments,targeted_structure),"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_ophys_experiments_one_id(bo_api):
    ophys_experiment_id = 502066273
    bo_api.get_ophys_experiments(ophys_experiment_id)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::OphysExperiment,rma::criteria,[id$in502066273],"
        "rma::include,"
        "well_known_files(well_known_file_type),targeted_structure,"
        "specimen(donor(age,transgenic_lines)),"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_experiment_container_metrics(bo_api):
    tid = 511510627
    bo_api.get_experiment_container_metrics(tid)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamExperimentContainerMetric,"
        "rma::criteria,[id$in511510627],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_experiment_containers(bo_api):
    tid = 511510753
    bo_api.get_experiment_containers(tid)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ExperimentContainer,rma::criteria,[id$in511510753],"
        "rma::include,ophys_experiments,isi_experiment,"
        "specimen(donor(age,transgenic_lines)),targeted_structure,"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_column_definitions(bo_api):      
    api_class_name = bo_api.quote_string('ApiTbiDonorMetric')
    bo_api.get_column_definitions(api_class_name=api_class_name)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiColumnDefinition,"
        "rma::criteria,[api_class_name$eq'ApiTbiDonorMetric'],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_list_column_definition_class_names(bo_api):
    bo_api.list_column_definition_class_names()
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiColumnDefinition,"
        "rma::options"
        "[only$eq'api_class_name'][num_rows$eq'all'][count$eqfalse]")


def test_get_stimulus_mappings_no_ids(bo_api):
    bo_api.get_stimulus_mappings()
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamStimulusMapping,"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_stimulus_mappings_one_id(bo_api):
    ids = 15
    bo_api.get_stimulus_mappings(stimulus_mapping_ids=ids)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamStimulusMapping,"
        "rma::criteria,[id$in15],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_stimulus_mappings_two_ids(bo_api):
    ids = [15, 43]
    bo_api.get_stimulus_mappings(stimulus_mapping_ids=ids)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamStimulusMapping,"
        "rma::criteria,[id$in15,43],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_cell_metrics_no_ids(bo_api):
    bo_api.get_cell_metrics()
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamCellMetric,"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_cell_metrics_one_ids(bo_api):
    tid = 517394843
    bo_api.get_cell_metrics(cell_specimen_ids=tid)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamCellMetric,"
        "rma::criteria,[cell_specimen_id$in517394843],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")


def test_get_cell_metrics_two_ids(bo_api):
    ids = [517394843,517394850]
    bo_api.get_cell_metrics(cell_specimen_ids=ids)
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::ApiCamCellMetric,"
        "rma::criteria,[cell_specimen_id$in517394843,517394850],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")
    
    
def test_filter_experiment_containers_no_filters(bo_api, mock_containers):
    containers = bo_api.filter_experiment_containers(mock_containers)
    assert len(containers) == 2


def test_filter_experiment_containers_depth_filter(bo_api, mock_containers):
    containers = bo_api.filter_experiment_containers(mock_containers,
                                                     imaging_depths=[100])
    assert len(containers) == 1


def test_filter_experiment_containers_structures_filter(bo_api, mock_containers):
    containers = \
        bo_api.filter_experiment_containers(
            mock_containers,
            targeted_structures=['CBS'])
    assert len(containers) == 1


def test_filter_experiment_containers_lines_all_filters(bo_api, mock_containers):
    containers = \
        bo_api.filter_experiment_containers(mock_containers,
                                            imaging_depths=[200],
                                            targeted_structures=['NBC'],
                                            transgenic_lines=['Don'])
    assert len(containers) == 1


def test_filter_ophys_experiments_no_filters(bo_api, mock_ophys_experiments):
    experiments = bo_api.filter_ophys_experiments(mock_ophys_experiments)
    assert len(experiments) == 2


def test_filter_ophys_experiments_container_id(bo_api, mock_ophys_experiments):
    experiments = bo_api.filter_ophys_experiments(mock_ophys_experiments,
                                                  experiment_container_ids=[1])
    assert len(experiments) == 1


def test_filter_ophys_experiments_stimuli(bo_api, mock_ophys_experiments):
    experiments = bo_api.filter_ophys_experiments(mock_ophys_experiments,
                                                  stimuli=['static_gratings'])
    assert len(experiments) == 1


def test_filter_cell_specimens(bo_api, mock_specimens):
    specimens = bo_api.filter_cell_specimens(mock_specimens)
    assert specimens == mock_specimens


def test_save_ophys_experiment_data(bo_api_save_ophys):
    bo_api = bo_api_save_ophys
    
    with patch('allensdk.config.manifest.Manifest.safe_mkdir') as mkdir:
        bo_api.retrieve_file_over_http = \
            MagicMock(name='retrieve_file_over_http')
        bo_api.save_ophys_experiment_data(1, '/path/to/filename')
        
        mkdir.assert_called_once_with('/path/to')
    
    bo_api.json_msg_query.assert_called_once_with(
        "http://testwarehouse:9000/api/v2/data/query.json?q="
        "model::WellKnownFile,"
        "rma::criteria,"
        "[attachable_id$eq1],well_known_file_type[name$eqNWBOphys],"
        "rma::options[num_rows$eq'all'][count$eqfalse]")
    bo_api.retrieve_file_over_http.assert_called_with(
        'http://testwarehouse:9000/url/path/to/file',
        '/path/to/filename')
